# bni-tools Copyright Notice
# =====================================
# 
# The bni-tools source code is copyrighted, but you can freely
# use and copy it as long as you don't change or remove any of the
# Copyright notices.  bni-tools is made available under the
# following open-source license terms:
# ----------------------------------------------------------------------
# bni-tools is Copyright (C) 2008-2010 Georg Steinkellner
#                         All Rights Reserved
# Permission to use, copy, modify, distribute, and distribute modified
# versions of this software and its built-in documentation for any
# purpose and without fee is hereby granted, provided that the above
# copyright notice appears in all copies and that both the copyright
# notice and this permission notice appear in supporting documentation,
# and that the names of the authors not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
# The authors DISCLAIM ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE AUTHORS BE 
# LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
# ----------------------------------------------------------------------
from pymol import cmd
from pymol import util
from pymol import preset
from chempy import cpv
import os
import tkSimpleDialog
__version__="0.27"
#
# Pymol Tools v.027
#
# AUTHOR: Georg Steinkellner
#
#.
#
sidebar_stdwidth=int(cmd.get("internal_gui_width"))
# set keys
try: 
  cmd.set_key('CTRL-C',cmd.create,(None,"sele",-1,-1,0))
  cmd.set_key('CTRL-A',cmd.select,("sele","visible"))
except:
  pass
def __init__(self):
   self.__class__._gui=self
   self.menuBar.addcascademenu('Plugin', 'Pymoltools', 'BNI Tools',
                               label='BNI Pymol Tools %s'%__version__,tearoff=True)
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Pseudo menue for clicking if clicking stucks 
   # Symmetry BEGIN
   #import info
   self.menuBar.addmenuitem('Pymoltools','separator','')
   self.menuBar.addcascademenu('Pymoltools','symmetry',statusHelp='Symmetry settings',
                      traverseSpec=None,
                      label='Symmetry ->'
                      )
   self.menuBar.addcascademenu('symmetry','symview','Create Symmetry',
                      label='Create Symmetry View ->'
                      )
   self.distances=["set",3.0,3.5,4.0,4.5,5.0,10.0]
   for item in self.distances:
        self.menuBar.addmenuitem('symview', 'command',
                      'Distance %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : fast_sym(cutoff=a,app=b,selection="sele"))#fast_sym(cutoff=item))
   self.menuBar.addmenuitem('symmetry', 'command',
                      'Set Surface to Symetry representation',
                      label='Set Surface to Symmetry',
                      command = lambda a="symmetry" : set_surface(mode=a))
   self.menuBar.addmenuitem('symmetry', 'command',
                      'Set Surface to standard representation',
                      label='Set Surface to standard',
                      command = lambda a="standard" : set_surface(mode=a))
   # Symmetry END
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Settings BEGIN
   self.menuBar.addcascademenu('Pymoltools',
                               'Settings',
			       'Get special settings',
                               label='Settings ->',tearoff=True
                               )
   #Side Bar settings
   self.menuBar.addcascademenu('Settings', 'SideBar',
                      'SideBar settings',
                      label='Side Bar ->',tearoff=True)
   self.menuBar.addcascademenu('SideBar', 'width',
                      'SideBar settings ->',
                      label='Width',tearoff=True)
   self.sidebar_lenght=["set",100,200,250,350,450]
   for item in self.sidebar_lenght:
        self.menuBar.addmenuitem('width', 'command',
                      'Width %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : runsidebar(set="width",value=a,app=b))#fast_sym(cutoff=item))
   self.menuBar.addmenuitem('SideBar', 'command',
                      'fit sidebar to names',
                      label='fit2name',
                      command = lambda a="fit1" : sidebar(set="width",value=a))
   self.menuBar.addmenuitem('SideBar', 'command',
                      'fit sidebar to titles',
                      label='fit2title',
                      command = lambda a="fit2" : sidebar(set="width",value=a))
   self.menuBar.addmenuitem('SideBar', 'command',
                      'sidebar off',
                      label='off',
                      command = lambda a="fit2" : sidebar(set="off",value=a))
   self.menuBar.addmenuitem('SideBar', 'command',
                      'sidebar on',
                      label='on',
                      command = lambda a="fit2" : sidebar(set="on",value=a))
   self.menuBar.addcascademenu('SideBar', 'overlay',
                      'Overlay settings ->',
                      label='Overlay')
   self.menuBar.addmenuitem('overlay', 'command',
                      'set  sidebar overlay on',
                      label='on',
                      command = lambda a="" : sidebar(set="overlayon",value=a))
   self.menuBar.addmenuitem('overlay', 'command',
                      'set sidebar overlay off',
                      label='off',
                      command = lambda a="" : sidebar(set="overlayoff",value=a))
   # Settings END
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Presets BEGIN      
   self.menuBar.addcascademenu('Pymoltools','presets','Preset Views',
                      label='Preset Views ->',tearoff=True
                      )
   self.menuBar.addmenuitem('presets', 'command',
                      'Track Mainchain',
                      label='Track MainChain',
                      command = lambda a="track_mainchain": new_views(preset=a))
   self.menuBar.addcascademenu('presets', 'HPvalue',
                      'Hydrophobic Values',
                      label='Hydrophobic Values ->'
                      )
   # 
   self.HPvalues=["KandD","Rose","GES"]
   for item in self.HPvalues:
        self.menuBar.addmenuitem('HPvalue', 'command',
                      'HPvalues %s'%item,
                      label='%s'%item,
                      command = lambda a=item : select_hp(hp_value=a))
   #
   self.menuBar.addmenuitem('presets', 'command',
                      'Surface Atoms',
                      label='Atoms for Surface Inspection',
                      command = lambda a="surf_rel_res": new_views(preset=a))
   # Presets END
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Load Begin
   self.menuBar.addcascademenu('Pymoltools','Load Files','Load Files',
                      label='Load Files ->',
                      tearoff=True)
   self.menuBar.addmenuitem('Load Files', 'command',
                      'Multiple Files',
                      label='Multiple Files',
                      command = lambda b=self: multi_pdb(b).fetchFileName())
   self.menuBar.addmenuitem('Load Files', 'command',
                      'Modeller Files',
                      label='Modeller Files',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="modeller"))
   self.menuBar.addmenuitem('Load Files', 'command',
                      'AutoDock Files',
                      label='AutoDock Files',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="autodock"))
   self.menuBar.addmenuitem('Load Files', 'command',
                      'Amber mimimization',
                      label='Amber minimization',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="ambermin"))
   self.menuBar.addmenuitem('Load Files', 'command',
                      'Delphi Map Files',
                      label='Delphi PHI,DX Map',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="delphi"))
   self.menuBar.addmenuitem('Load Files', 'command',
                      'Casox Map',
                      label='Casox Map',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="casox"))
   self.menuBar.addmenuitem('Load Files', 'command',
                      'One Object',
                      label='Multiple Files into states',
                      command = lambda b=self: multi_pdb(b).fetchFileName(ext="oneobject"))
   #Load End
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Edit BEGIN
   self.menuBar.addcascademenu('Pymoltools',
                               'EditAtom',
                               'Edit Atoms',
                               label='Edit ->',
                               tearoff=True)
   self.menuBar.addmenuitem('EditAtom',
                            'command',
                            'His2',
                            label="HIS --> HID,HIE,HIP",
                            command= lambda b="his2hid": his_handle(b))
   self.menuBar.addmenuitem('EditAtom',
                            'command',
                            'Hid2',
                            label="HID,HIE,HIP --> HIS",
                            command= lambda b="hid2his": his_handle(b))
   self.menuBar.addmenuitem('EditAtom',
                            'command',
                            'polyalanine',
                            label="Poly-Alanine Chain",
                            command= lambda : polyala("sele",name="polyala"))
   self.menuBar.addmenuitem('EditAtom',
                            'command',
                            'MES->MET',
                            label="MES-->MET",
                            command= lambda : altermse("sele"))
   self.menuBar.addmenuitem('EditAtom',
                            'command',
                            'delalt',
                            label="del alternates",
                            command= lambda : delalternate("sele"))
   # Edit END
   # Create BEGIN
   self.menuBar.addmenuitem('Pymoltools','separator','')
   self.menuBar.addcascademenu('Pymoltools',
                               'Create',
                               'Create',
                               label='Create ->',
                               tearoff=True)
   #Ray BEGIN
   self.menuBar.addcascademenu('Create','Ray','Ray Setting',
                     label='ray ->',tearoff=True
                     )
   self.ray_settings=["set","presentation","journal single","journal double","poster","hires"]
   for item in self.ray_settings:
        self.menuBar.addmenuitem('Ray', 'command',
                      'Ray %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : _set_bni_ray(setting=a,app=b))#fast_sym(cutoff=item))
   #self.menuBar.addmenuitem('Ray',
   #                         'command',
   #                         'His2',
   #                         label="HIS --> HID,HIE,HIP",
   #                         command= lambda b="his2hid": his_handle(b))
   #Ray End
   #Plane begin
   self.menuBar.addcascademenu('Create','GetPlane','Get Plane',
                     label='Plane ->',tearoff=True
                     )
   self.plane_settings=["set","large","medium","small"]
   for item in self.plane_settings:
        self.menuBar.addmenuitem('GetPlane', 'command',
                      'Plane %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : get_plane_handler(action=a,app=b))
   #Plane end
   #Box begin
   self.menuBar.addcascademenu('Create','GetBox','Get Box',
                     label='Box ->',tearoff=True
                     )
   self.box_settings=["set","large","medium","small"]
   for item in self.box_settings:
        self.menuBar.addmenuitem('GetBox', 'command',
                      'Box %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : get_box_handler(action=a,app=b))
   #Box End
   #Set Surface BEGIn
   self.menuBar.addcascademenu('Create','SetSurface','Set surface',
                     label='Surface ->',tearoff=True
                     )
   self.surf_settings=["Methionine","Include","Exclude","Hide","Show"]
   for item in self.surf_settings:
        self.menuBar.addmenuitem('SetSurface', 'command',
                      'Surface %s'%item,
                      label='%s'%item,
                      command = lambda (a,b)=(item,self) : get_surface_handler(action=a,app=b))
   #Set Surface End
   # Create END
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Delete BEGIN
   self.menuBar.addcascademenu('Pymoltools', 'delete',
                      'Delete all',
                      label='Delete enabled',
                      tearoff=True)
   self.menuBar.addmenuitem('delete', 'command',
                      'enabled',
                      label='all',
                      command = lambda : del_enabled('all'))
   self.menuBar.addmenuitem('delete', 'command',
                      'objects',
                      label='objects',
                      command = lambda : del_enabled('obj'))
   self.menuBar.addmenuitem('delete', 'command',
                      'selections',
                      label='selections',
                      command = lambda : del_enabled('selections'))
   self.menuBar.addcascademenu('Pymoltools', 'invert',
                      'invert all',
                      label='Invert enabled/disabled',
                      tearoff=True)
   self.menuBar.addmenuitem('invert', 'command',
                      'all',
                      label='all',
                      command = lambda : inv_enabled('all'))
   self.menuBar.addmenuitem('invert', 'command',
                      'objects',
                      label='objects',
                      command = lambda : inv_enabled('obj'))
   self.menuBar.addmenuitem('invert', 'command',
                      'selections',
                      label='selection',
                      command = lambda : inv_enabled('selections'))
   #combine  BEGIN
   self.menuBar.addcascademenu('Pymoltools','Combine','Combine selections',
                     label='Combine sele..',tearoff=True
                     )
   self.combine_sele=["all","enabled","disabled"]
   for item in self.combine_sele:
        self.menuBar.addmenuitem('Combine', 'command',
                      'combine %s'%item,
                      label='%s'%item,
                      command = lambda a=item : combine_sele(check=a))
   #combine END
   # Delete END
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Create Pseudo Atom for centers
   self.menuBar.addmenuitem('Pymoltools', 'command',
                      'Create PseudoCenter Atom',
                      label='PseudoCenter Atom',
                      command = lambda : get_center_atom(selection='(sele)',enabled="1",onlysele="1"))
   self.menuBar.addmenuitem('Pymoltools','separator','')
   # Get sequence
   self.menuBar.addcascademenu('Pymoltools', 'Sequences',
                      'get sequence',
                      label='Get sequences',
                      tearoff=True)
   self.menuBar.addmenuitem('Sequences', 'command',
                      'Fasta',
                      label='FASTA',
                      command = lambda : get_sequence(selection='(sele)',format="fasta"))
   self.menuBar.addmenuitem('Sequences', 'command',
                      'pir',
                      label='pir',
                      command = lambda : get_sequence(selection='(sele)',format="pir"))
   self.menuBar.addmenuitem('Sequences', 'command',
                      'modeller',
                      label='modeller',
                      command = lambda : get_sequence(selection='(sele)',format="modeller"))
   self.menuBar.addcascademenu('Sequences', 'Normal',
                      'get normal sequence',
                      label='List',
                      tearoff=True)
   self.menuBar.addmenuitem('Normal', 'command',
                      'Residue',
                      label='Normal',
                      command = lambda : get_sequence(selection='(sele)',format="normal"))
   self.menuBar.addmenuitem('Normal', 'command',
                      'Residue',
                      label='bfac per res',
                      command = lambda : get_sequence(selection='(sele)',format="normalb"))
   self.menuBar.addmenuitem('Normal', 'command',
                      'Residue',
                      label='occ per res',
                      command = lambda : get_sequence(selection='(sele)',format="normalq"))
   self.menuBar.addmenuitem('Normal', 'command',
                      'Residue',
                      label='get atoms',
                      command = lambda : get_sequence(selection='(sele)',selecting='atom',format="normal"))
   # Print PDB line
   self.menuBar.addmenuitem('Pymoltools', 'command',
                      'Get PDB lines',
                      label='Get PDB String',
                      command = lambda : get_pdb_lines(selection='(sele)',state="0",enabled="1"))
   cmd.extend('fast_sym',fast_sym)
   cmd.extend('del_enabled',del_enabled)
   cmd.extend('del_sym',del_sym)
   cmd.extend('new_views',new_views)
   cmd.extend('new_name',rename_object)
   cmd.extend('get_pdb_lines',get_pdb_lines)
   cmd.extend('mse',mse)
   cmd.extend('get_box',get_box)
   cmd.extend('get_axes',get_axes)
   cmd.extend('get_point',get_point)
   cmd.extend('get_atom',get_atom)
   cmd.extend('get_center',getCenter)
   cmd.extend('get_center_atom',get_center_atom)
   cmd.extend('check_names',check_names)
   cmd.extend('AA',AA)
   cmd.extend('sidebar',sidebar)
   cmd.extend("multi_pair_align",multi_pair_align)
   cmd.extend("get_plane",get_plane)
   cmd.extend("get_polyala",polyala)
   cmd.extend("his_handle",his_handle)
   cmd.extend("get_sequence",get_sequence)
   cmd.extend("bni_ray",bni_ray)
   cmd.extend("combine_sele",combine_sele)
def testi(item):
        print item
def combine_sele(selections=None,check="enabled",name="sele"):
   '''Combine selections. 
   USAGE: combine_sele(selections=None,check="enabled",name="sele")
            selections: "sele1 sele2 sele3": add additinal selections
                 check:           "enabled": add only enabled selections
                                      "all": add all  selections
			         "disabled": add only disabled selectins
			                 -1: do not look for selections
                  name:                sele: specify name of combined selection
   PYMOL: combine_sele selections=None,check=-1,name="sele"
   '''
   selections=_remove_brakets(selections)
   enabled=str(_remove_brakets(check))
   name=str(_remove_brakets(name))
   if enabled=="enabled":# combine enabled selections
     sele_names=set(cmd.get_names("selections",enabled_only=1))
   elif enabled=="all":# combine all selections
     sele_names=set(cmd.get_names("selections",enabled_only=0))
   elif enabled=="disabled":# combine not enabled selections
     sele1=set(cmd.get_names("selections",enabled_only=1))
     sele2=set(cmd.get_names("selections",enabled_only=0))
     sele_names=sele2-sele1
   elif enabled=="-1":# do not check only use selections input
     sele_names=[]
   sele_names=list(sele_names)
   if selections:
      addsele=selections.split()
      for item in addsele:
         sele_names.append(item)
   print "sele_names",sele_names
   if sele_names:
      cmd.select(name,selection="("+" or ".join(sele_names)+")")
      #cmd.set_name("_bni"+name,name)
   else:
      cmd.select(name,selection=None)
      #cmd.set_name("_bni"+name,name)
   print "# BNI - Tools v. %s"%__version__
   print "# Combined selections: %s"%sele_names
   print "# to (%s)"%name
def _set_bni_ray(setting="set",app=None,async=1):
   '''Run sidebar input
   '''
   async=int(_remove_brakets(async))
   width=100
   dpi=200
   width_unit="mm"
   if setting=="set" and app:
      value=tkSimpleDialog.askstring('Width(mm) resolution(dpi)',
                                     'Please specify width of picture and resolution',
                                                      parent=app.root)
      value=_remove_brakets(value)
      if value:
        value = value.split()
      else:
        return
      if len(value)==1:
         width=value[0]
      elif len(value)==2:# width / dpi
         width=value[0]
         dpi=value[1]
      elif len(value)>=3:#width width_unit 
         width=value[0]
         width_unit=value[1]
         dpi=value[2]
      else:
        print "Wrong input. Set at least width."
	return
      try:
        float(width)
      except ValueError:
        print "Width has to be a number."
	return
      units=["cm","mm","inch","dots"]
      if not width_unit.lower().strip() in units:
            print "Unit has to be either %s "%(" or ".join(units)) 
	    return
      try:
         int(dpi)
      except ValueError:
         print "DPI has to be an integer number."
	 return
   elif setting=="poster":
     print "# POSTER SETTING"
     width=30
     width_unit="cm"
     dpi=300
     print "# width: %s %s"%(width,width_unit)
     print "# resolution:  %sdpi"%dpi
   elif setting=="journal single":
     print "# JOURNAL SETTING single column"
     width=86
     width_unit="mm"
     dpi=400
     print "# width: %s %s"%(width,width_unit)
     print "# resolution:  %sdpi"%dpi
   elif setting=="journal double":
     print "# JOURNAL SETTING double column"
     width=178
     width_unit="mm"
     dpi=400
     print "# width: %s %s"%(width,width_unit)
     print "# resolution:  %sdpi"%dpi
   elif setting=="presentation":
     print "# PRESENTATION SETTING"
     width=150
     width_unit="mm"
     dpi=150
     print "# width: %s %s"%(width,width_unit)
     print "# resolution:  %sdpi"%dpi
   elif setting=="hires":
     print "# HIRES SETTING"
     width=120
     width_unit="mm"
     dpi=1200
     print "# width: %s %s"%(width,width_unit)
     print "# resolution:  %sdpi"%dpi
   if async==1:
     print "# Ray is working in the background"
   bni_ray(width,width_unit=width_unit,dpi=dpi,async=async)
def bni_ray(width,name_png=None,dpi="300",width_unit="cm",**arg):
   '''Ray specified with dpi and width.
   USAGE: bni_ray(width,dpi="300",width_unit="cm",name_png=None,**arg)
	   width: width of ray picture in defined units
	   name_png:     ./name.png: write png file with ./name.png
	           :               : do not write png just do ray
	   width_unit:    : unit of width
			cm: centi meter  
		      inch: inch
			mm: milli meter
		      dots: (normal behaviour of ray)
	   dpi:        300:dots per inch (resolution)
	   **arg: additinal arguments for ray
		  height: integer :{default: 0 (current)}
	       antialias: integer :{default: -1 (use antialias setting)}
		   angle: float   :y-axis rotation for stereo image generation
		                   {default: 0.0}
		   shift: float   : x-axis translation for stereo image generation
				   {default: 0.0}
	        renderer: -1      :respectively
		           0      :default
			   1      :built-in
			   2      :pov-ray, or dry-run {default: 0}
		   async:  0      : do not run in background  thread
			   1      : background thread
   PYMOL: bni_ray width,name_png,dpi="300",width_unit="cm",**arg e.g. angle=3.0
   '''
   width=float(_remove_brakets(width))
   width_unit=_remove_brakets(width_unit)
   dpi=int(_remove_brakets(dpi))
   if width_unit=="cm":
      calc_res=round(width*dpi/2.54)
   elif width_unit=="mm":
      calc_res=round(width*dpi/25.4)
   elif width_unit=="inch":
      calc_res=round(width*dpi)
   elif width_unit=="dots":
      calc_res=int(width)
   else:
     print "Unit not recodgnized: %s"%width_unit
     return
   print "# BNI - Tools v. %s"%__version__
   print "# %d (dots in x direction)"%(calc_res)
   print "# Size(x): %s %s, Resolution: %sdpi"%(width,width_unit,dpi)
   print "# [Size(y): calculated automatically.]"
   cmd.ray(width=calc_res,**arg)
   if name_png is not None:
       name_png=_remove_brakets(name_png)
       name=os.path.abspath(os.path.normpath(name_png))
       cmd.png(name,dpi=dpi)
       print "# saved as: %s "%name
def his_handle(fromto="his2hid",selection="(sele)",hisH_dic=None):
   '''Alter HIS.
   fromto:"his2hid":  HIS to HIP HID HIE
         :"hid2his":  HIS HIP HIE to HIS
   hisH_dic: dictionary of his hydrogen naming convention
             e.g. {"HID":["HD1","H0X"],
                   "HIE":["HE2","H01"]}
   selection: (sele):
   '''
   selection=_remove_brakets(selection)
   fromto=_remove_brakets(fromto)
   try:
      cmd.count_atoms(selection)
   except:
          print "Please select at least one residue. (sele)"
          return
   if fromto=="his2hid":
        print "BNI - Tools v. %s"%__version__
        print "---------------------"
        print "Rename HIS to HID HIE HIP"
        print "Hydrogens have to be set correctly"
        print "at the HIS residues "
        print "HD1 ?         --> HID"
        print "HE2 ?         --> HIE"
        print "HD1 and HE2 ? --> HIP"
        print "---------------------"
        #This script renames HIS to HIP HID HIE from reduce pdb output
        #it overwrites the input file
        #CYS to CYX is not performed!!
        allhis=cmd.count_atoms("(r. HIS) and n. CA")
        #hisH_dic naming convention   [reduce,pymol]
        if not hisH_dic:
            hisH_dic={"HID":["HD1"],
                      "HIE":["HE2","H01"]}
        cmd.select("HIP","(byres ((n. %s) and r. HIS) and byres ((n. %s) and r. HIS))"%(",".join(hisH_dic.get("HID")),",".join(hisH_dic.get("HIE"))))
        cmd.alter("(HIP)","resn='HIP'")
        cmd.sort()
        cmd.select("HID","(byres ((n. %s) and r. HIS))"%(",".join(hisH_dic.get("HID"))))
        cmd.alter("(HID)","resn='HID'")
        cmd.sort()
        cmd.select("HIE","(byres ((n. %s) and r. HIS))"%(",".join(hisH_dic.get("HIE"))))
        cmd.alter("(HIE)","resn='HIE'")
        cmd.sort()
        print "%s HIS residues"%allhis
        print "-------------------------------"
        print "%s HID"%cmd.count_atoms("(r. HID and n. CA)")
        print "%s HIE"%cmd.count_atoms("(r. HIE and n. CA)")
        print "%s HIP"%cmd.count_atoms("(r. HIP and n. CA)")
        print "-------------------------------"
        print "%s HIS residues altered"%cmd.count_atoms("((r. HID,HIE,HIP) and n. CA)")
        print "-------------------------------"
   elif fromto=="hid2his":
        print "BNI - Tools v. %s"%__version__
        print "--------------------"
        print "Rename HID HIE HIP to HIS"
        print "--------------------"
        allhidhiehis=cmd.count_atoms("((r. HID,HIE,HIP) and n. CA)")
        print "%s HID"%cmd.count_atoms("(r. HID and n. CA)")
        print "%s HIE"%cmd.count_atoms("(r. HIE and n. CA)")
        print "%s HIP"%cmd.count_atoms("(r. HIP and n. CA)")
        cmd.select("HIS","(byres (r. HIE,HIP,HID))")
        cmd.alter("(HIS)","resn='HIS'")
        cmd.sort()
        allhis=cmd.count_atoms("(r. HIS) and n. CA")
        print "%s HIS residues"%allhis
        print "-------------------------------"
        print "%s HID,HIE,HIP residues altered"%allhidhiehis
        cmd.sort()
def polyala(selection="(sele)",name=None):
        '''Get Poly Alanine of selected residues.
        USAGE:  polyala(selection=None,name=None)
                  selection: selection of residues
                  name:      name of created polyalanine
                             if no name is given the original input
                             will be altered to polyalanine
        PYMOL:  get_polyala (selection),[name]
                  selection: selection of residues
                  name:      name of created polyalanine
                             if no name is given the original input
                             will be altered to polyalanine
        ADDITIONAL: if "name" is specified the structure looks wired because of the
                    lack of bonds between the altered alanine residues.
                    If you want to see the bonds you can save and reopen the struture.
                    (Pymol recognize the new bonds after loading)
        '''
        if selection is None:
          print "Please select at least one residue."
          return
        tmpname2="_polyala2"
        tmpname3="_polyala3"
        selection=_remove_brakets(selection)
	try:
	  cmd.count_atoms(selection)
	except:
	  print "Emty selection. Select at least one atom."
	  return
        if name<>None:
	   if name in cmd.get_names():
	      cmd.delete(name)
           cmd.select(tmpname2,"((byobj %s) and not (byres %s))"%(selection,selection))
           cmd.create(tmpname3,"((byres %s) and (n. CB,O,CA,N,C) and not hetatm and not solvent)"%(selection))
           cmd.alter(("%s and not r. GLY")%tmpname3,"resn='ALA'")
           cmd.sort()
           cmd.create(name,("(%s or %s)"%(tmpname3,tmpname2)))
           cmd.delete(tmpname2)
           cmd.delete(tmpname3)
        else:
           cmd.remove("((byres %s) and not (n. CB,O,CA,N,C) and not hetatm and not solvent)"%selection)
           cmd.select(tmpname3,"((byres %s) and (n. CB,O,CA,N,C) and not hetatm and not solvent)"%(selection))
           cmd.alter(("%s and not r. GLY")%tmpname3,"resn='ALA'")
           cmd.sort()
           cmd.delete(tmpname3)
def point(p):
    cgo=cgo_dic()
    COLOR=cgo["COLOR"]
    SPHERE=cgo["SPHERE"]
    x, y, z = p
    return [SPHERE, float(x), float(y), float(z), 0.2]
def line(p1, p2):
    cgo=cgo_dic()
    CYLINDER=cgo["CYLINDER"]
    x1, y1, z1 = p1
    x2, y2, z2 = p2
    #return [CYLINDER, float(x1), float(y1), float(z1), float(x2), float(y2), float(z2), 0.1, 1, 1, 1, 1, 1, 1]
    return []
def plane(c1, c2, c3, c4, norm,invert=0,sphere=1):
    cgo=cgo_dic()
    BEGIN=cgo["BEGIN"]
    COLOR=cgo["COLOR"]
    TRIANGLE_STRIP=cgo["TRIANGLE_STRIP"]
    TRIANGLES=cgo["TRIANGLES"]
    NORMAL=cgo["NORMAL"]
    VERTEX=cgo["VERTEX"]
    END=cgo["END"]
    planeObj = []
    if sphere==1:
     planeObj.extend(point(c1))
     planeObj.extend(point(c2))
     planeObj.extend(point(c3))
     planeObj.extend(point(c4))
    planeObj.extend([BEGIN, TRIANGLES])
    corners=[c2,c1,c3,c4,c3,c1]
    if invert==1:
       normal=norm
    elif invert==2:
       normal=[i*(-1) for i in norm]
    else:  
         for corner in corners:
            planeObj.append(VERTEX)
            planeObj.extend(corner)
         planeObj.append(END)
         return planeObj   
    for corner in corners:
        planeObj.append(NORMAL)
        planeObj.extend(normal)
        planeObj.append(VERTEX)
        planeObj.extend(corner)
    planeObj.append(END)
    return planeObj
def planeFromPoints(point1, point2, point3, facetSize,invert=0,sphere=1):
    v1 = cpv.normalize(cpv.sub(point2, point1))
    v2 = cpv.normalize(cpv.sub(point3, point1))
    normal = cpv.cross_product(v1, v2)
    v2 = cpv.cross_product(normal, v1)
    x = cpv.scale(v1, facetSize)
    y = cpv.scale(v2, facetSize)
    center = point2
    corner1 = cpv.add(cpv.add(center, x), y)
    corner2 = cpv.sub(cpv.add(center, x), y)
    corner3 = cpv.sub(cpv.sub(center, x), y)
    corner4 = cpv.add(cpv.sub(center, x), y)
    return plane(corner1, corner2, corner3, corner4, normal, invert,sphere)
def get_plane_handler(selection="(sele)",app=None,action="set",name="plane",size=100,invert=2,sphere=0):
     '''Get a plane using get_plane GUI.
     '''
     action=_remove_brakets(action)
     name=check_names(name)
     size=size
     invert=str(invert)
     sphere=str(sphere)
     if action=="set" and app:
      value=tkSimpleDialog.askstring('Plane settings',
                                     'Specify [size A] [invert 2/1/0]  [sphere 1/0]',
                                                      parent=app.root)
      value=_remove_brakets(value)
      if value:
        value = value.split()
      else:
        return
      if len(value)==1:
         size=value[0]
      elif len(value)==2:# width / dpi
         size=value[0]
         invert=str(value[1])
      elif len(value)>=3:#width width_unit 
         size=value[0]
         invert=str(value[1])
         sphere=str(value[2])
      else:
        print "Wrong input. Set at least size."
	return
      try:
        float(size)
      except ValueError:
        print "Size has to be a number."
	return
      units=["0","1","2"]
      if not invert.lower().strip() in units:
            print "Invert has to be either %s "%(" or ".join(units)) 
	    return
      units=["1","0"]
      if not sphere.lower().strip() in units:
            print "Sphere has to be either %s "%(" or ".join(units)) 
	    return
     elif action=="small":
	size=10
        invert=2
        sphere=0
     elif action=="medium":
        size=50
        invert=2
        sphere=0
     elif action=="large":
        size=100
        invert=0
        sphere=0
     get_plane(selection,name=name,size=size,invert=invert,sphere=sphere)
def get_plane(selection,name="plane",size=100,invert=0,sphere=1):
    '''Get a plane from 3 atom selection.
    USAGE: get_plane(selection,name="plane",size=100,invert=0,sphere=1)
             name     :        :name of the plane
             selection:        :selection containing three atoms
             size     : int    :size of the plane
             invert   : int    : 2: invert normal for lightning
                                 1: do not invert normal
                                 0: do not create normal
             sphere   : int    : 1: draw spheres on edges of the plane
                               : 0: do not draw spheres
    PYMOL: get_plane selection,name,size,invert,sphere
    '''
    name=_remove_brakets(name)
    selection=_remove_brakets(selection)
    size=_remove_brakets(size)
    invert=_remove_brakets(invert)
    size=int(size)
    invert=int(invert)
    coordinates=[]
    try:
        mol = cmd.get_model(selection)
    except:
        print "***\n*** Empty selection\n***"
        return
    else:
        if len(mol.atom)<>3:
          print "***\n*** Please select exactely three atoms\n***"
          return
        for a in mol.atom:
            coordinates.append(a.coord)
        point1 = coordinates[0]
        point2 = coordinates[1]
        point3 = coordinates[2]
        plane = planeFromPoints(point1, point2, point3, int(size),invert,sphere)
        planeName = name
        cmd.load_cgo(plane, planeName)
        cmd.show("cgo", planeName)
        #print coordinates
#plane end
def _get_list(input,message="Error"):
   '''Remove brakets and return list.
      Returns a list from input,
      else a error massage is printed
   '''
   input=_remove_brakets(input)
   if type(input) is str:
       try:
         input=map(int,input.split(","))
       except TypeError:
         print "%s:\n%s"%(message,mobile)
         return
       except ValueError: 
          input=input.split(",")
   return list(input)
def multi_pair_align(mobile,fixed,selection="all2first",sel="atomsel",fix=None,mob=[None,]):
   '''Align multiple structures pairwise using pair_wise wizard.
   USAGE: multi_pair_align(mobile=[1,],fixed=[1,],sel="atomsel",selection="all2first",fix=None,mob=[None,])
             mobile    : [1,5,7]             : atom Nr. list to align from 
                                             : mobile structures
                       : [atom1,atom2,atomx] : list of atom selections
             fixed     : [1,5,7]             : atom Nr. list to align from
                                             : target structure (first entry)
                       : [atom1,atom2,atomx] : list of atom selections
             sel       : "atomnr"            :selection is atom number
                         "atomsel"           :selection is atom selection (std)
             selection : "all2first"         : selection of structures 
                                               first is fixed structure
                                               all structures are aligned
                                               to the first one
                       : "all2fix"           : specify fixed structure
                                               all structures are aligned to the
                                               selected "fix" structure
                       : "mob2fix"           : specify fixed and mobile struxtures
                                               "mob" structures are aligned to the 
                                               "fix" structure
             fix       : "name"              : fix structure
             mob       : ["name1,"name2",..] : mobile structures
   PYMOL: multi_pair_align mobile, fixed, sel, selection, fix , mob                    
   '''
   mobile=_get_list(mobile,message="Error in creating mobile atom align list")
   fixed=_get_list(fixed,message="Error in creating fixed atom align list")
   if not mobile and not fixed:
         print "Error in reading mobile and fixed atoms.\nmobile:%s\nfixed:%s"%(mobile,fixed)
         return
   selection=_remove_brakets(selection)
   fix=_remove_brakets(fix)
   sel=_remove_brakets(sel)
   mob=_get_list(mob)
   print "mobile:",mob
   if selection=="all2first":
        struc=cmd.get_names()
        struc_mob=struc[1:]
        try:
         struc_fix=struc[0]
        except IndexError:
           print "No target structure"
           return
        del struc
   elif selection=="fix":
        struc=cmd.get_names()
        struc_fix=fix
        struc.pop(struc.index(struc_fix))
        struc_mob=struc[:]
        del struc
   elif selection=="mob2fix":
        struc_mob=mob 
        struc_fix=fix 
   else:
        struc_fix=None
        struc_mob=[]
   ##cmd.feedback("disable","selector","everything")  
   print "Multiple Pair Alignment"
   print "with pair_align wizard"
   print "by BNI-Tools v.%s"%__version__
   for item in struc_mob:
     print "__________________________"
     print "Target: %-20s"%(struc_fix)
     print "Mobile: %-20s"%(item)
     cmd.wizard("pair_fit")
     cmd.refresh_wizard()
     print "     __________________________"
     atom_numbers=[]
     cmd.feedback("disable","all","output")
     sel_err=[]
     for mob_atom,fix_atom in zip(mobile,fixed):
          atom_numbers.append("     %-10s:%-10s"%(fix_atom,mob_atom))
          cmd.select('sele','none')
          #check if input are index atom numbers or selected atoms
          if sel == "atomnr":
           cmd.select('sele',"(%s`%d) and not (%s`%d)"%(item,mob_atom,item,mob_atom))
           if cmd.count_atoms("sele") <> 1:
              sel_err.append("Not a single atom selection: %s'%s"%(item,mob_atom))
              continue
           cmd.get_wizard().do_select('''sele''')
           cmd.select('_indicate_pf',"((((_indicate_pf) or ((%s`%d))) and not ((((%s`%d))) and (_indicate_pf))))"%(struc_fix,fix_atom,struc_fix,fix_atom))
           if cmd.count_atoms("_indicate_pf") <> 2:
              sel_err.append("Not a single atom selection: %s'%s"%(struc_fix,fix_atom))
              continue
           cmd.get_wizard().do_select('''_indicate_pf''')
           cmd.get_wizard().fit()
          else:
           print "mobile:"
           print "%s and (%s)"  %(item,mob_atom)
           cmd.select('_mpasele',"((((sele) or ((%s and (%s)))) and not ((((%s and (%s)))) and (sele))))"%(item,mob_atom,item,mob_atom))
           print "atom1",cmd.count_atoms("sele")
           if cmd.count_atoms("sele") <> 1:
              sel_err.append("Not a single atom selection: %s and %s"%(item,mob_atom))
              continue
           cmd.get_wizard().do_select('''sele''')
           cmd.select('_indicate_pf',"((((_indicate_pf) or ((%s and (%s)))) and not ((((%s and (%s)))) and (_indicate_pf))))"%(struc_fix,fix_atom,struc_fix,fix_atom))
           print "atom2",cmd.count_atoms("_indicate_pf")
           if cmd.count_atoms("_indicate_pf") <> 2:
              sel_err.append("Not a single atom selection: %s and %s"%(struc_fix,fix_atom))
              continue
           cmd.get_wizard().do_select('''_indicate_pf''')
           try:
              cmd.get_wizard().fit()  
           except:
                sel_err.append("Cannot perform fit, some selections may be wrong")
                sel_err.append("Target structure:%s"%struc_fix)
                sel_err.append("Atom: "%fix_atom)
                sel_err.append("Mobile structure(s) "%item)
                sel_err.append("Atom:"%mob_atom)
                continue
     cmd.feedback("enable","all","output")
     print "     __________________________"
     print "     %-10s|%-10s"%("Target","Mobile")
     print "     %-10s|%-10s"%("AtomNr.","AtomNr.")
     print "\n".join(atom_numbers)
     print "\n".join(sel_err)
     cmd.feedback("disable","all","debugging")
     print "-----------------"
     cmd.set_wizard()
   cmd.feedback("enable","selector","everything")
def _maxnamelenght(types):
   '''Get max lenght of names.
   types: 1 : get name lenght
          2 : get name and title lenght
   '''
   stdwidth=sidebar_stdwidth
   multipl=11.0 
   addpixel=30.0 #add to width
   if types==1:
      names=cmd.get_names()
      try:
        maxlenght=max([len(i) for i in names])
      except ValueError:
        return stdwidth
      try:
         namelenght=int(round(float(maxlenght*multipl+addpixel+0.5)))
      except ValueError:
         return stdwidth
      if namelenght<stdwidth:
         namelenght=stdwidth
      return namelenght   
   elif types==2:
      names=cmd.get_names()
      titles=[]
      for tit in names:
         try:
           title=cmd.get_title(tit,cmd.get_state())
           if title==None:
              title=""
         except AttributeError:
            titles.append("")
         titles.append(title)
      if len(names)<>len(titles):
         print "name and title mismatch/nnames:%s/ntitles:%s"%(names,titles)
      else:
         try:
            maxlenght=max([len(u)+len(v) for u,v in zip(names,titles)])
            #maxlenght=maxlenght+2*multipl
         except ValueError:
            return stdwidth
      try:
         namelenght=int(round(float(maxlenght*multipl+0.5)))
      except ValueError:
         return stdwidth
      if namelenght<stdwidth:
         namelenght=stdwidth
      return namelenght
def runsidebar(set="",value=None,app=None):
   '''Run sidebar input
   '''
   if value=="set" and app:
      value=tkSimpleDialog.askstring('Input Sidebar Value',
                                                      'Please specify a Value e.g 250',
                                                      parent=app.root)
   sidebar(set="width",value=value)
def sidebar(set="",value=None):
   '''Sidebar Settings.
   AUTHOR: Georg Steinkellner
   USAGE: sidebar set,value
      set: 
             "on"   :  set sidebar on
             "off"  :  set sidebar off
             "width":  set sidebar to value
             "size" :  set hight of each sidebar entry
             "overlayon": sidebar overlay on
             "overlayoff": sidebar overlay off
      value:
             value : int : value of sidebar
   '''
   valuedic={}
   if set=="on":
      cmd.set("internal_gui","on")
   elif set=="off":
      cmd.set("internal_gui","off")
   elif set=="overlayon":
      try:
         cmd.set("internal_gui_mode",1)
      except:
         pass
   elif set=="overlayoff":
      try:
         cmd.set("internal_gui_mode",0)
      except:
         pass
   elif set=="width":
      if type(value)<>type(None):
         if value=="fit1":#fit to names
            fit=_maxnamelenght(1)
            valuedic.update({"width":fit})
         elif value=="fit2":#fit to names and titles
            fit=_maxnamelenght(2)
            valuedic.update({"width":fit})
         else:            
          try:
            value=int(float(value))
            valuedic.update({"width":value})
          except:
            print "Wrong value:",value
            return
      else:
         return
      try:
       cmd.set("internal_gui_width",valuedic.get("width",250))
       print "Set sidebar to %s"%valuedic.get("width",250)
      except:
         return
   elif set=="size":
      cmd.set("internal_gui_control_size",valuedic.get("size",18))
def _check_selection(selection="(sele)"):
   '''Check if selection contains atoms.
   '''
   try:
       return cmd.count_atoms(selection)
   except:
       return 0
def get_surface_handler(selection="(sele)",action="mse",app=None):
   '''Surface representation GUI using mse.
   '''
   selection=_remove_brakets(selection)
   action=_remove_brakets(action)
   len_atoms=_check_selection(selection)
   if action=="Methionine" or action =="mse":
        mse(sele="r. MSE",stype="-1",ssurface="1")
        print "# BIN - Tools v. %s"%__version__
        print "# Methionine surface representation on."
	return
   if len_atoms==0:
      print "# No atoms in selection %s. Select at least 1 atom."%selection
      return
   elif action=="Include":
        mse(sele=selection,stype="-1",ssurface="1")
        print "# BIN - Tools v. %s"%__version__
        print "# Included %s atom(s) to surface calc."%len_atoms
   elif action=="Exclude":
        mse(sele=selection,stype="-1",ssurface="0")
        print "# BIN - Tools v. %s"%__version__
        print "# Excluded %s atom(s) from surface calc."%len_atoms
   elif action=="Show":
        mse(sele=selection,stype="-1",ssurface="1")
        print "# BIN - Tools v. %s"%__version__
        print "# Show %s atom surfaces."%len_atoms
   elif action=="Hide":
        mse(sele=selection,stype="-1",ssurface="2")
        print "# BIN - Tools v. %s"%__version__
        print "# Hide %s atom surfaces."%len_atoms
def delalternate(sele="sele"):
   '''Delete alternates.
   '''
   sele=_remove_brakets(sele)
   selection="("+sele+" and not (alt '' or alt A))"
   try:
    len_atoms=cmd.count_atoms(selection)
    len_residues=cmd.count_atoms("(byres "+selection+") and n. CA and not e. CA and (alt '' or alt A)")
   except:
    print "No residues to delete in selection"
    return
   cmd.remove(selection)
   cmd.sort()
   print "# BIN - Tools v. %s"%__version__
   print "# Deleted %d alternate atoms in %d residues."%(len_atoms,len_residues)
def altermse(sele="(sele)"):
   '''Edit MSE to MET permanantly.
   '''
   sele=_remove_brakets(sele)
   selection="("+sele+" and r. MSE and n. SE)"
   cmd.alter(selection=selection,expression="name='S'",quiet=1,space=None)
   cmd.sort()
   cmd.flag(25,"("+sele+" and r. MSE)","clear")
   cmd.sort()
   selection="("+sele+" and r. MSE)"
   mses=cmd.count_atoms(selection+" and n. CA and not e. CA")
   cmd.alter(selection=selection,expression="resn='MET'",quiet=1,space=None)
   print "# BIN - Tools v. %s"%__version__
   print "# Changed %d MES residues to MET."%mses
   cmd.sort()
def mse(sele="r. MSE",stype="-1",ssurface="1"):
   '''Handle MSE atom surface representation (Seleno- Methionine).
   AUTHOR: Georg Steinkellner
   USAGE:  mse sele="r. MSE" stype=-1 ssurface=1
           sele:     selection : standard is "r. MSE"
           stype:    1: set MSE type from HETATM to ATOM
                     0: set MSE type from ATOM to HETATM
                    -1: do nothing (standard)
           ssurface: 1: set surface representation to 1 for MSE (create surface) (standard)
                     2: set surface representation to 2 for MSE (dont show but include)
                     0: set surface representation to 0 for MSE (exclude from surface)
                        this is standard within pymol
                    -1: do nothing
   '''
   sele=_remove_brakets(sele)
   if stype=="1":
     cmd.alter("%s"%sele,'type="ATOM"')
   elif stype=="0":
      cmd.alter("%s"%sele,'type="HETATM"')
   else:
      pass
   if ssurface=="1":
     cmd.flag(25,sele,"clear")
     cmd.flag(24,sele,"clear")
   elif ssurface=="2":
      cmd.flag(25,sele,"clear")
      cmd.flag(24,sele,"set")
   elif ssurface=="0":
      cmd.flag(25,sele,"set")
      cmd.flag(24,sele,"clear")
   else:
      pass
   cmd.sort()
def _redirect_alter(selection,property,addprint=1,command="alter"): 
    '''redirect standart out temporarily to get cmd.alter command variables.
     selection:
     property: index
     addprint:1: additional print to stdout
     command:"alter": use the cmd.alter command
             "iterate": use the cmd.iterate command (v.1.2r1 also needs print redirection!)
    '''
    if command=="alter":
       command=cmd.alter
    elif command=="iterate":
       command=cmd.iterate
    values=[]
    class MyRedirect:
      def __init__(self,stdout):
        self.stdout=stdout
      def write(self, s):
        if s.strip():
           values.append(s)
        if addprint==1:
           self.stdout.write(s)
    old_stdout = sys.stdout
    sys.stdout = MyRedirect(sys.stdout)
    command(selection,property) # index of the atoms is now in a list(values) as string
    sys.stdout = old_stdout
    #print values
    return values
def _seq_format(selection="sele",wrap=60,format="fasta",gap=1,gap_sign="-"):
             '''sequence output.
             format: pir
                     modeller
                     fasta
             wrap: sequence chars in one line
             gap:      1:
                       0:
             gap_sign: -:
                       .:
                       _:
                   lower:gap AA are written in lower case
	     returns (fromatted sequences,gap_residues)
             '''
             output_format="'%s;%s'%(resn,resi)"
             chains=cmd.get_chains(selection)
             allnu=[]
             names=cmd.get_names()
             exc=HP()
	     nr_out=[]
             for obj in names:
              for chain in chains:
	       if chain=='':
	         chain="''"
               select_new="%s and "%obj+selection+" and c. %s"%chain
               u=_redirect_alter(select_new,output_format,addprint=0,command="alter")
               #print select_new
               if not u:
                 continue
               u=[((_remove_brakets(i.split(";")[0]),_remove_brakets(i.split(";")[1]))) for i in u if type(i) is type("string")] 
               lenght=len(u)
               aa=[a[0] for a in u]
               nr=[a[1] for a in u]
               aa=exc.three2one(aa)
               if not len(aa)==len(nr):
                  print "AA and NR do not have the same lenght!"
                  print "output in line 2 of pir format may be incorrect."
               nu="".join(aa)
               if chain=='':
                 chain="."
               #compare with whole chain to insert gaps as "-"
               if gap==1:
                  select_gap="(bychain "+select_new+") and n. CA and not e. CA and (alt '' or alt A)" 
                  u_gap=_redirect_alter(select_gap,output_format,addprint=0)
                  u_gap=[((_remove_brakets(i.split(";")[0]),_remove_brakets(i.split(";")[1]))) for i in u_gap if type(i) is type("string")] 
                  lenght_gap=len(u_gap)
                  aa_gap=[a_gap[0] for a_gap in u_gap]
                  nr_gap=[a_gap[1] for a_gap in u_gap]
                  aa_gap=exc.three2one(aa_gap)
                  if not len(aa_gap)==len(nr_gap):
                    print "AA_gap and NR_gap do not have the same lenght!"
                    print "output may be incorrect."
                  #nu_gap="".join(aa_gap)
                  #compare and create new seq with gaps
                  new_seq=[]
                  #print nr_gap,aa_gap
                  #print nr,aa
		  nr_out.append("# Residue from - to")
		  nr_out.append("# for %s:%s:%d (out of %d)"%(obj,chain,lenght,len(nr_gap)))
		  contains_gap=0
		  new_nr=[]
                  for number,amino in zip(nr_gap,aa_gap):
                     if number in nr:
                         new_seq.append(amino)
			 new_nr.append((number,amino))
                     else:
		         if new_nr:
		            nr_out.append("%1s %4s - %1s %4s : (%3d AA)"%(new_nr[0][1],new_nr[0][0],new_nr[-1][1],new_nr[-1][0],len(new_nr))) 
                            contains_gap=1
			    new_nr=[]
			 if gap_sign=="lower":
			   new_seq.append(amino.lower())
			 else:
                           new_seq.append(gap_sign)
		  if contains_gap==0:# no gaps
	             nr_out.append("%1s %4s - %1s %4s : (%3d AA)"%(aa_gap[0],nr_gap[0],aa_gap[-1],nr_gap[-1],len(nr_gap)))
		  elif new_nr: 
		               nr_out.append("%1s %4s - %1s %4s : (%3d AA)"%(new_nr[0][1],new_nr[0][0],new_nr[-1][1],new_nr[-1][0],len(new_nr))) 
			       new_nr=[]
			       if gap_sign=="lower":
			          new_seq.append(amino.lower())
			       else:
                                  new_seq.append(gap_sign)
		  nu="".join(new_seq)
               if format=="modeller":
                  allnu.append(">P1;%s"%(obj))
		  label="structureX"
		  source="source"
		  rfac=-1.0
		  resolution=-1.0
                  if gap<>1:
                    allnu.append("%s:%s:%s:%d:%s:%d:SeqLenght %d:%s:%.2f:%.2f"%(label,obj,chain,int(nr[0]),chain,int(nr[-1]),lenght,source,rfac,resolution))
                  else:
                    allnu.append("%s:%s:%s:%d:%s:%d:SeqLenght %d:%s:%.2f:%.2f"%(label,obj,chain,int(nr_gap[0]),chain,int(nr_gap[-1]),lenght,source,rfac,resolution))
                  #every "wrap" res return
                  nu=_seq_wrap(nu,wrap)
                  nu[-1]+="*"
               elif format=="pir":
                  allnu.append(">P1;%s"%(obj))
                  allnu.append("")
                  #every "wrap" res return
                  nu=_seq_wrap(nu,wrap)
                  nu[-1]+="*"
               elif format=="fasta":
                  if gap==1:
                         allnu.append(">%s:%s:%d out of %d:"%(obj,chain,lenght,len(nr_gap)))
                  else:
                     allnu.append(">%s:%s:%d"%(obj,chain,lenght))
                  #every "wrap" res return
                  nu=_seq_wrap(nu,wrap)
               allnu.extend(nu)
             return allnu,nr_out
def _seq_wrap(seq,wrap=60):
    '''Sequence wrapper.
    Wraps a sequence into a list.
    seq: string input
    wrap: 60: chars per list entry
    '''
    slice_begin=0
    slice_end=wrap
    seq_list=[]
    while 1:
        seq_slice=seq[slice_begin:slice_end]
        if len(seq_slice)<wrap:
               seq_list.append(seq_slice)
               break
        seq_list.append(seq_slice)
        slice_begin=slice_end
        slice_end+=wrap
    return seq_list
def _mw(values=[]):
    '''Return MW  and Variance and len.
    '''
    sum1=sum(values)
    num=len(values)
    try:
      mean=float(sum1)/float(num)
      std=(sum([(a-mean)**2 for a in values])/(num))**0.5
    except ZeroDivisionError:
       mean=0.
       std=0.
       num=0.
    return mean,std,num
def get_sequence(selection="sele",selecting="residue",format="normal",name=None,wrap="60",gap="1",gap_sign="-"):
    '''Print selected residues.
         USAGE: get_sequence(selection="sele",selecting="residue",format="normal",name=None,wrap="60")
                selection: sele    : selection of residues or atoms.
                selecting: residue: get all residues of selection (without alternates)
                              atom: get all atoms of selection
                             chain: get all chains of selection
                    name: None    : name of selection for output
                   format: normal: object: chain: residue-residue number
		          normalb: calc bfac per residue
			  normalq: calc occ per residue
                            fasta: amino acid one letter code in fasta format
                              pir:    amino acid one letter code in pir format
                         modeller:    amino acid one letter code in Modeller pir format
                   wrap:  60: for sequence output wrap at 60 residues
                   gap:  1: add gaps between selected amino acids
                         0: do not add gaps
                   gap_sign: -   : char of gap 
		                  "lower": gap amino acids are written as lower case
         PYMOL:  get_sequence selection=sele,selecting=residue,format=normal,wrap=60,gap_sign=lower
         ADDITIONAL: alternates are not considered! eighter alternate A or alterneta "" are selected
    ''' 
    format=_remove_brakets(format)
    selecting=_remove_brakets(selecting)
    selection=_remove_brakets(selection)
    wrap=int(_remove_brakets(wrap))
    gap=int(_remove_brakets(gap))
    gap_sign=_remove_brakets(gap_sign)
    try:
       a=cmd.count_atoms(selection)
    except:
       a=None
    if not a:
        print "Select at least one atom."
        return
    if name is None:
       name=selection
    prelist=[]
    prelist.append("# BNI-Tools v. %s"%__version__)
    prelist.append("# ---------------")
    prelist.append("# Output for %s"%name)
    prelist.append("# ---------------")
    if selection=="enabled":
        pass
    elif selection=="enabled1":
        pass
    else:
       output_format='"%s:%s: %s-%s"%(model,chain,resn,resi)'
       if selecting=="residue":
          selection="(byres "+selection+")"
	  selection="("+selection+" and n. CA and not e. CA )" 
	  selection="("+selection+" and (alt '' or alt A))"
          if format=="normal" or format=="normalb" or format=="normalq":
             output_format='"%s:%s: %s-%s "%(model,chain,resn,resi)'
             u=_redirect_alter(selection,output_format,addprint=0)
             u=[_remove_brakets(i) for i in u]
	     if format=="normalb" or format=="normalq":
		if format=="normalb":
		  val="b"
		  output_format='"%4.2f"%(b)'
		else:
		  val="q"
		  output_format='"%4.2f"%(q)'
		resi=[]
	        for residue in u:
		  residue=residue.split(":")
	          selection="("+residue[0]+" and c. "+residue[1]+" and r. "+residue[2].split("-")[0]+" and i. "+residue[2].split("-")[1]+")"
		  selection="("+selection+" and (alt '' or alt A))" 
		  a=_redirect_alter(selection,output_format,addprint=0)
		  a=[_remove_brakets(i) for i in a]
		  val_list=[float(i) for i in a]
		  residue.extend(["%s= %5.2f"%(val,_mw(val_list)[0]),"%s= %5.2f"%("s",_mw(val_list)[1]),"atoms %2d"%_mw(val_list)[2]])
		  resi.append(":".join(residue))
		u=resi
             print "\n".join(u)
             return u,[]
          elif format=="fasta" or format=="pir" or format=="modeller":
             (allnu,nr_gap)=_seq_format(selection=selection,
                               wrap=wrap,
                               gap=gap,
                               gap_sign=gap_sign,
                               format=format)
	     if gap==1:
	        prelist.extend(nr_gap)
             prelist.append("# SEQUENCE FORMAT: %s"%format)
	     prelist.append("# ---------------------------")
             prelist.extend(allnu)
             print "\n".join(prelist)
             return allnu,nr_gap
          else:
             output_format='"%s"%(resn)'
             u=_redirect_alter(selection,output_format,addprint=0)
             #print u
             u=[_remove_brakets(i) for i in u]
             print "\n".join(u)
             return u,[]
       elif selecting=="atom":
           selection=selection
           output_format='"%s:%s %s-%s %s"%(model,chain,resn,resi,name)'
           u=_redirect_alter(selection,output_format,addprint=0)
           #print u
           u=[_remove_brakets(i) for i in u]
           print "\n".join(u)
           return u,[]
       elif selecting=="chain":
           u=cmd.get_chains(selection)
           print "\n".join(u)
           return u,[]
       #cmd.alter(selection,output_format)
class Sym:
        def __init__(self):
                self.org_cc = []  
                self.sym_cc = []  
                self.org_sel = [] 
                self.sym_sel = [] 
                self._cutoff=0.0
                self.name_sym_obj=""
        def _setcutoff(self,restrict=6.0):
                '''Check if cutoff has to be restricted.
                '''
                if self._cutoff>restrict:
                        self._cutoff=restrict
        def create_sym(self,cutoff=4.0,selection="sele"):
                '''Create smmetry mates of selection.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                USAGE: 
                        create_sym([cutoff=4.0[,selection="sele"]])
                RETURNS: 
                        selection_name of symetry involved atoms of symetry mates
                '''
                selection=_remove_brakets(selection)
                self._cutoff=cutoff
                #restrict selection cutoff to a reasonable distance (max. 6.0 A)
                self._setcutoff()
                try:
                  self.name_sym_obj=cmd.get_object_list(selection)[0]
                except:
                  self.name_sym_obj=""
                cmd.select("_sym_sele","(%s)"%selection)
                cmd.create("_sym","_sym_sele")
                cmd.symexp("s_%s"%self.name_sym_obj,"_sym",selection,cutoff=cutoff,segi=1)
                cmd.delete("_sym_sele")
                cmd.delete("_sym")
                select_name="sym_%s_%1.1f"%(self.name_sym_obj,self._cutoff)
                cmd.select(select_name,"(%s around %1.1f) and not ((byobj %s) and not %s)"%(selection,self._cutoff,selection,selection),enable=1)
                symetrys=[r for r in cmd.get_names() if "s_%s"%self.name_sym_obj in r]
                colors=2
                for item in symetrys:
                        try:
                                util.cba(colors,item)
                        except:
                                pass
                        colors+=1
                return select_name
        def create_org(self,cutoff=4.0,sele=None):
                '''Create selection of orig atoms involved within symmetry.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                USAGE:
                        create_org([cutoff=4.0
                                   [,sele="s_*"
                                   ]])
                '''
                if sele is None:
                   sele="s_%s*"%self.name_sym_obj
                self._cutoff=cutoff
                #restrict selection cutoff to a reasonable distance (max. 6.0 A)
                self._setcutoff()
                cmd.select("_sym",sele)
                #restrict selection cutoff to a reasonable distance (max. 6.0 A)
                select_name="org_%s_%1.1f"%(self.name_sym_obj,self._cutoff)
                cmd.select(select_name,"(_sym around %1.1f)"%self._cutoff,enable=1)
                cmd.delete("_sym")
                return select_name
        def show_org(self,select_name,show_name="surface",cutoff=4.0,surf_transparency=0.0):
                '''Create different views of symmetry involved selection.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                USAGE: show_org(select_name,
                                [show_name="surface"
                                [,cutoff=4.0,
                                [,surface_transparency=0.0,
                                ]])
                                show_name:"sticks"
                                          "surface"
                '''
                select_name=_remove_brakets(select_name)
                self._cutoff=cutoff
                #restrict selection cutoff to a reasonable distance (max. 6.0 A)
                self._setcutoff()
                cmd.enable(select_name)
                cmd.show(show_name,"org_%s_%1.1f"%(self.name_sym_obj,self._cutoff))
                if show_name=="surface":
                        cmd.set("transparency",surf_transparency)
                cmd.disable("org_%s_%1.1f"%(self.name_sym_obj,self._cutoff))
        def show_sym(self,select_name,show_name="surface",cutoff=4.0,surf_transparency=0.0,object=None):
                '''Create different views of symmetry involved selection.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                USAGE: show_sym(select_name,
                                [show_name="surface"
                                [,cutoff=4.0,
                                [,surface_transparency=0.0,
                                [,sele="s_*"]]])
                                show_name:"sticks"
                                          "surface"
                '''
                select_name=_remove_brakets(select_name)
                self._cutoff=cutoff
                #restrict selection cutoff to a reasonable distance (max. 6.0 A)
                self._setcutoff()
                cmd.enable(select_name)
                cmd.show(show_name,"sym_%s_%1.1f"%(self.name_sym_obj,self._cutoff))
                if object is None:
                   object="s_%s*"%(self.name_sym_obj)
                if show_name=="surface":
                        cmd.set("transparency",surf_transparency,object)
                cmd.disable("sym_%s_%1.1f"%(self.name_sym_obj,self._cutoff))
        def get_model(self,selection=""):
                '''Return Model of selection.
                '''
                selection=_remove_brakets(selection)
                try:
                        mol = cmd.get_model(selection)
                except:
                        print "Please specify a selection"
                        return 0
                return mol
class HP:
        def __init__(self):
                self.one_letter={"ALA":["A"],
                             "ARG":["R"],
                             "ASN":["N"],
                             "ASP":["D"],
                             "ASX":["B"],
                             "CYS":["C"],
                             "CYH":["C"],#protonated cys
                             "CYX":["C"],#cystein
                             "GLN":["Q"],
                             "GLU":["E"],
                             "GLY":["G"],
                             "GLX":["Z"],
                             "HIS":["H"],
                             "HIP":["H"],#protonated
                             "HID":["H"],#h on delta
                             "HIE":["H"],#h on epsilon
                             "ILE":["I"],
                             "LEU":["L"],
                             "LYS":["K"],
                             "MET":["M"],
                             "MSE":["M"],#Seleno- Methionin
                             "PHE":["F"],
                             "PRO":["P"],
                             "SER":["S"],
                             "THR":["T"],
                             "TRP":["W"],
                             "TYR":["Y"],
                             "VAL":["V"],
                             "UNK":["X"]}
                self.three_letter={     "A":["ALA"],
                                "R":["ARG"],
                                "N":["ASN"],
                                "D":["ASP"],
                                "B":["ASX"],
                                "C":["CYS","CYH","CYX"],
                                "Q":["GLN"],
                                "E":["GLU"],
                                "G":["GLY"],
                                "Z":["GLX"],
                                "H":["HIS","HIE","HIP","HID"],
                                "I":["ILE"],
                                "L":["LEU"],
                                "K":["LYS"],
                                "M":["MET"],
                                "MSE":["MSE"],#seleno-methionine
                                "F":["PHE"],
                                "P":["PRO"],
                                "S":["SER"],
                                "T":["THR"],
                                "W":["TRP"],
                                "Y":["TYR"],
                                "V":["VAL"],
                                "X":["UNK"]}
        def one2three(self,aa_list):    
                '''Get one letter code from AA list.
                '''
                u=[]
                tmp_list=[self.three_letter.get(l,["UNK"]) for l in aa_list]
                for item in tmp_list:
                        u.extend(item)
                return u
        def three2one(self,aa_list):
                '''Get one letter code from AA list.
                '''
                u=[]
                tmp_list=[self.one_letter.get(l,["X"]) for l in aa_list]
                for item in tmp_list:
                        u.extend(item)
                return u
        def get_HP_set(self,hp_value="GES"):
                '''Get one letter code for hydrophobic,neutral,hydrophilic.
                USAGE:  get_HP_set(hp_value="GES")
                                hp_value: "GES"
                                          "KandD"
                                          "Rose"
                RETURNS: (hydrophobic_AA,neutral_AA,hydrophilic_AA)
                '''
                #GES
                #Kyte & Doolittle
                #Rose
                hydrophobic = {"GES":
                                ["F","M","I","L","V","C","W","A","T","G","S","MSE"],
                               "KandD":
                                ["A","C","I","L","M","F","V","MSE"],
                                "Rose":
                                ["C","I","L","M","F","W","V","MSE"]
                              }
                hydrophilic = {"GES":["E","K","D","R"],
                                "KandD":
                                ["R","N","D","Q","E","H","K"],
                                "Rose":
                                ["R","N","D","Q","E","K","P","S"]
                              }
                neutral     = {"GES":["P","Y","H","Q","N"],
                               "KandD":
                               ["G","P","S","T","W","Y"],
                               "Rose":
                               ["A","G","H","T","Y"]
                               }
                return (self.one2three(hydrophobic.get(hp_value)),self.one2three(neutral.get(hp_value)),self.one2three(hydrophilic.get(hp_value)))
class set_views:
        def __init__(self):
                self.selection="all"
        def mainchain(self,sele="all",obj_name="MC_track"):
                '''Create mainchain track in a Mainchain Object.
                mainchain(sele="all")
                '''
                self.selection=_remove_brakets(sele)
                try:
                 cmd.create(obj_name,"(%s) and (n. C,N,O,CA)"%(self.selection))
                 #create polar contacts of mainchain with preset and rename the contacts
                 cmd.dist("%s_mcp_cont"%obj_name[:2],"(%s) and not (solvent or (polymer and not name n,o,h))"%self.selection,"(%s) and not (solvent or (polymer and not name n,o,h))"%self.selection,quiet=1,mode=2,label=0,reset=1)
                 cmd.enable("%s_mcp_cont"%obj_name[:2])
                 try:
                        cmd.color(8,"%s_mcp_cont"%obj_name[:2])#color magenta
                 except:
                        pass
                 cmd.show_as("sticks",obj_name)
                 cmd.set("stick_radius",0.20,"all")
                 cmd.set("stick_radius",0.35,obj_name)
                 cmd.set("stick_transparency",0.5,obj_name)
                 cmd.dist("%s_scp_cont"%obj_name[:2],"(%s)"%self.selection,"(%s) and not (polymer and name n,o,h)"%self.selection,quiet=1,mode=2,label=0,reset=1)
                 cmd.enable("%s_sc_scp_cont"%obj_name[:2])
                except:
                 print "Can not create track. (no structure enabled?)"
                cmd.color("white",obj_name)
        def sidechain(self,sele="all"):
                '''Create sidechain.
                '''
                self.selection=_remove_brakets(sele)
                cmd.select("Sidechain","(%s) and not n. C,N.O"%self.selection)
                cmd.show("lines","Sidechain")
        def surface_atoms_preset(self,sele="all"):
                '''Get different important atoms.
                '''
                self.selection=_remove_brakets(sele)
                try:
                        cmd.set("surface_color",-1,self.selection)
                        cmd.select("HIS","(%s) and (r. HIS,HIE,HID,HIP)"%self.selection)
                        cmd.select("HIS_ND1NE2","(%s) and (r. HIS,HIE,HID,HIP) and (n. ND1,NE2)"%self.selection)
                        cmd.color("gray","(%s)"%self.selection)
                        try:
                                util.cba(29,"HIS_ND1NE2")
                        except:
                                pass
                        cmd.select("ASP","(%s) and (r. ASP)"%self.selection)
                        cmd.select("ASP_OD12","(%s) and (r. ASP) and (n. OD1,OD2)"%self.selection)
                        try:
                                util.cba(29,"ASP_OD12")
                        except:
                                pass
                        cmd.select("GLU","(%s) and (r. GLU)"%self.selection)
                        cmd.select("GLU_OE12","(%s) and (r. GLU) and (n. OE1,OE2)"%self.selection)
                        try:
                                util.cba(29,"GLU_OE12")
                        except:
                                pass
                        cmd.select("ARG","(%s) and (r. ARG)"%self.selection)
                        cmd.select("ARG_NH12","(%s) and (r. ARG) and (n. NH1,NH2)"%self.selection)
                        try:
                                util.cba(29,"ARG_NH12")
                        except:
                                pass
                        cmd.select("LYS","(%s) and (r. LYS)"%self.selection)
                        cmd.select("LYS_NZ","(%s) and (r. LYS) and (n. NZ)"%self.selection)
                        try:
                                util.cba(29,"LYS_NZ")
                        except:
                                pass
                        cmd.select("CYS","(%s) and (r. CYS,CYH,CYX)"%self.selection)
                        cmd.select("CYS_SG","(%s) and (r. CYS,CYH,CYX) and (n. SG)"%self.selection)
                        try:
                                util.cba(29,"CYS_SG")
                        except:
                                pass
                except:
                        print "Could not create preset. (No Structure enabled?)"
def new_views(preset="track_mainchain"):
        '''Create Preset Views.
        '''
        #look if object is molecule and not a previous track run
        names=[i for i in cmd.get_names("objects",enabled_only=1) if cmd.get_type(i)=="object:molecule"]
        if preset=="track_mainchain":
                #create for all enabled objects
                obj_name="MC_track"
                try:
                        names.remove(obj_name)
                except ValueError:
                        pass
                selection=_remove_brakets(' or '.join(names))
                v=set_views()
                v.mainchain(selection,obj_name=obj_name)
        if preset=="surf_rel_res":
                selection=_remove_brakets(' or '.join(names))
                v=set_views()
                v.surface_atoms_preset(selection)
def set_surface(mode="standard"):
        '''Set surface to standard values.
        AUTHOR:
                Georg Steinkellner, 11/2006
        USAGE: 
                set_surface(mode="standard")
                        mode: "standard"
                              "symmetry"
        '''
        if mode=="standard":
                transparency=0.0
                surfmode=-1
                selection="all"
                cmd.set("surface_color",surfmode,selection)
                cmd.set("transparency",transparency,selection)
        elif mode=="symmetry":
                surf_transparency=0.5
                selection="all"
                set_surf_color(color="blue",sele="all")
                set_surf_color(color="white",sele="s_*")
                cmd.set("transparency",surf_transparency,selection)
        else:
                pass
def inv_enabled(selection='all'):
        '''Invert all enabled objects.
        AUTHOR:
                Georg Steinkellner, 12/2006
        USAGE: 
                inv_enabled selection="obj" # delete all enabled objects
                inv_enabled "selections" # delete all enabled selections
                inv_enabled "all" # delete all enabled 
        '''
        #invert
        selection=_remove_brakets(selection)
        print "invert"
	vers=cmd.get_version()[1]
	if vers >= 1.21 and selection=="obj":
	    selection="objects"
        set_enabled = set(cmd.get_names(selection,enabled_only=1))
        set_all = set(cmd.get_names(selection,enabled_only=0))
        enabled=set_enabled
        disabled=set_all-set_enabled
        #print "disabled",disabled
        #cmd.disable(item)
        #cmd.enable(item)
        try:
                cmd.disable(' and '.join(enabled))
        except IndexError:
                pass
        try:
                cmd.enable(' and '.join(disabled))
        except IndexError:
                pass
def del_enabled(selection="obj"):
        '''Delete all enabled objects.
        AUTHOR:
                Georg Steinkellner, 11/2006
        USAGE: 
                del_enabled selection="obj" # delete all enabled objects
                del_enabled "selections" # delete all enabled selections
                del_enabled "all" # delete all enabled 
        '''
        selection=_remove_brakets(selection)                
	vers=cmd.get_version()[1]
	if vers >= 1.21 and selection=="obj":
	    selection="objects"
        for item in cmd.get_names(selection,enabled_only=1):
                cmd.delete(item)
def rename_object(old_name="sele",new_name="new"):
        '''Rename any object without wizard.
        AUTHOR:
                Georg Steinkellner, 11/2006
        (The wizard cannot rename ramps and other stuff.)
        '''
        old_name=_remove_brakets(old_name)
        new_name=_remove_brakets(new_name)
        cmd.set_name(old_name,new_name)
def set_surf_color(color="-1",sele="all"):
        '''Set surface color idependently from structure.
        AUTHOR:
                Georg Steinkellner, 11/2006
        color: "-1" : default
        color: white
                "1"
                "2"
                "3"
                .
        '''
        color=_remove_brakets(color)
        sele=_remove_brakets(sele)
        try:
           color=int(color)
        except ValueError:
           pass
        cmd.set("surface_color",color,sele)
def get_pdb_lines(selection="all",state="0",enabled="0"):
        '''Get pdb lines from selection.
        AUTHOR:
                Georg Steinkellner, 11/2006
        USAGE: get_pdb_lines(selection="all",state="0",enabled="0")
        PYMOL: get_pdb (select),state
                        selection:   object or selection
                        state:       state
                        enabled:     1: get only enabled items
                                     0: get all items
        '''
        selection=_remove_brakets(selection)
        if selection <> "all":
           names=[selection]
        else:
         names=[i for i in cmd.get_names(selection,enabled_only=int(enabled)) if cmd.get_type(i)=="object:molecule" or cmd.get_type(i)=="selection"]
        cnt=sum([cmd.count_atoms(i) for i in names])
        print "REMARK ________-Cut-Here-_________"
        print "REMARK Created by BNI Pymol Plugin v.%s"%(__version__)
        print "REMARK %s %s ATOMS"%(','.join(names),cnt)
        print "REMARK BEGIN PDB LINES FOR : %s"%(','.join(names))
        for item in names:
                print get_pdb(selection=item,state=int(state))
                print "REMARK END   PDB LINES FOR : %s"%item
        print "REMARK ________-Cut-END-_________"
def get_pdb(selection="all",state="0"):
        '''Get pdb lines from selection.
        AUTHOR:
                Georg Steinkellner, 11/2006
        USAGE: get_pdb(selection="all",state=0)
                        selection:   object or selection
                        state:       state
        '''
        selection=_remove_brakets(selection)
        pdbline=cmd.get_pdbstr(selection=selection,state=int(state))
        return pdbline
def del_sym(select_name="s_*"):
                '''Delete created symmetry mates.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                '''
                cmd.delete(select_name)
def select_hp(hp_value="KandD"):
                '''Select HP and color HP Values.
                '''
                u=HP()
                (hydrophobic,neutral,hydrophilic)=u.get_HP_set(hp_value=hp_value)
                print "HP_Values %s"%hp_value
                print "HP_pos (Hydrophobic) brown"
                print ('\t'+', '.join(hydrophobic))
                print "HP_ntr (Neutral) green"
                print ('\t'+', '.join(neutral))
                print "HP_neg (Hydrophilic) blue"
                print ('\t'+', '.join(hydrophilic))
                cmd.select("HP_pos",'v. and r. %s'%(','.join(hydrophobic)))
                cmd.color("brown","(HP_pos)")
                cmd.select("HP_ntr",'v. and r. %s'%(','.join(neutral)))
                cmd.color("green","(HP_ntr)")
                cmd.select("HP_neg",'v. and r. %s'%(','.join(hydrophilic)))
                cmd.color("blue","(HP_neg)")
def cgo_dic():
       cgo_dic={
        'POINTS'             : 0.0,
        'LINES'              : 1.0,
        'LINE_LOOP'          : 2.0,
        'LINE_STRIP'         : 3.0,
        'TRIANGLES'          : 4.0,
        'TRIANGLE_STRIP'     : 5.0,
        'TRIANGLE_FAN'       : 6.0,
        #QUADS              : 7.0
        #QUAD_STRIP         : 8.0
        #POLYGON            : 9.0
        'STOP'               :  0.0,
        'NULL'               :  1.0,
        'BEGIN'              :  2.0,
        'END'                :  3.0,
        'VERTEX'             :  4.0,
        'NORMAL'             :  5.0,
        'COLOR'              :  6.0,
        'SPHERE'             :  7.0,
        'TRIANGLE'           :  8.0,
        'CYLINDER'           :  9.0,
        'LINEWIDTH'          : 10.0,
        'WIDTHSCALE'         : 11.0,
        'ENABLE'             : 12.0,
        'DISABLE'            : 13.0,
        'SAUSAGE'            : 14.0,
        'CUSTOM_CYLINDER'    : 15.0,
        'SHAPE_VERTEX'       : 16.0,
        'SHAPE_COLOR'        : 17.0,
        'SHAPE_NORMAL'       : 18.0,
        'FONT'               : 19.0,
        'FONT_SCALE'         : 20.0,
        'FONT_VERTEX'        : 21.0,
        'FONT_AXES'          : 22.0,
        'CHAR'               : 23.0,
        'ALPHA'              : 25.0, #transparency 1.00-0.01
        'LIGHTING'           : float(0x0B50)}
       return cgo_dic
def _remove_brakets(string):
   '''Remove brakets and quotation marks from string.
   '''
   if type(string) is not str:
      return string
   string=string.replace("'","")
   string=string.replace('"',"")
   string=string.replace("(","")
   string=string.replace(")","")
   return string
def get_center_atom(selection='all',name=None,enabled="1",onlysele="0"):
   '''Get pseudo center of selection.
   USAGE: get_center_atom(selection,[name="center"])
   PYMOL: get_center_atom selection,[name=center])
   '''
   selection=_remove_brakets(selection)
   if selection <> "all":
           names=[selection]
           enabled="0"
   else:
         if onlysele=="0":
           names=[i for i in cmd.get_names(selection,enabled_only=int(enabled)) if cmd.get_type(i)=="object:molecule" or cmd.get_type(i)=="selection"]
         elif onlysele=="1":
           names=[i for i in cmd.get_names(selection,enabled_only=int(enabled)) if cmd.get_type(i)=="selection"]
         cnt=sum([cmd.count_atoms(i) for i in names])
   py_name=_remove_brakets(name)
   for item in names:
     print item
     center=getCenter(source=item)
     if center:
       if not name:
           py_name="c_"+item
       get_atom(name=py_name,coord=center)
     else:
       print "No Center to create."
def getCenter(source=''):
    '''
    DESCRIPTION:
        calculate the coordinate center of a selection
    AUTHOR:
        Karl Gruber, 11/2006
    USAGE:
        get_center selection
    '''
    source=_remove_brakets(source)
    try:
        mol = cmd.get_model(source)
    except:
        print "***\n*** Empty selection\n***"
        print getCenter.__doc__
    else:
        center = [0.,0.,0.]
        num_atom=0
        for a in mol.atom:
            num_atom=num_atom+1
            center[0]=center[0]+a.coord[0]
            center[1]=center[1]+a.coord[1]
            center[2]=center[2]+a.coord[2]
        if num_atom>0:
          center[0]=center[0]/num_atom
          center[1]=center[1]/num_atom
          center[2]=center[2]/num_atom
	else:
	  print "No Atoms in selection."
          return
        print "gridcenter %8.3f %8.3f %8.3f\n" % (center[0], center[1], center[2])
        return (center[0], center[1], center[2])
def get_atom(name="atom",coord="(0.0,0.0,0.0)"):
   '''Create dummy atom on specified coordinate.
   USAGE: get_atom(name="atom",coord="(0.0,0.0,0.0)")
   PYMOL: get_atom name="atom",coord=(0.0,0.0,0.0)
   '''
   get_point(name,coord,radius="0.5",atom="1")
def check_names(name=''):
        '''Check if name is already in pymol viewer and create new name.
         USAGE: check_names(name='')
         OUTPUT: new name string: name_01
        '''
        names=cmd.get_names()
        if not names:
          return name
        i=1
        name_check=name
        check_again=1
        while check_again:
          if check_again==0:
           break
          for item in names:
            if item.lower()==name_check.lower():
              try:#add additional number
                  a+=1
                  counter=int(name.split("_")[-1])
                  counter+=a
                  name_check="_".join(name.split("_")[:-1])+"_"+str(counter).zfill(2)
                  check_again=1
                  i=0 #set count for new number to 0
                  break
              except (IndexError,ValueError):#add new number
                 i+=1
                 name_check=name+"_"+str(i).zfill(2)
                 a=0 # set count for additional number to 0
                 check_again=1
                 break
            else:
               check_again=0
        return name_check
def get_point(name="point",coord="(0.0,0.0,0.0)",radius="0.5",atom="0",state="new"):
        '''Create CGO Sphere point.
        USAGE: get_point(name="point",
                         coord="(0.0,0.0,0.0)",
                         radius="0.5",
                         atom="0")
        PYMOL: get_point name="point",             #Name of object
                         coord="(0.0,0.0,0.0)",    #coord of object
                         radius="0.5",             #radius of cgo
                         atom="0"                  #1: create dummy atom instead of cgo
                         state="new"               #"new": create unique name if name already exists
        '''
        radius=float(radius)
        if state=="new":
           name=check_names(name=name)
        if type(coord) is str:
            coord=_remove_brakets(coord)
            coord=coord.split(",")
            coord=tuple(map(float,coord))
            print coord
        else:
           pass
        if int(atom)==0:
           dic=cgo_dic()
           BEGIN=dic['BEGIN']
           TRIANGLES=dic['TRIANGLES']
           COLOR=dic['COLOR']
           SPHERE=dic['SPHERE']
           LINES=dic['LINES']
           NORMAL=dic['NORMAL']
           VERTEX=dic['VERTEX']
           END=dic['END']
           p=[SPHERE,
              coord[0],coord[1],coord[2],radius]
           cmd.load_cgo(p,name)
        elif int(atom)==1:
           record_name='ATOM  '
           atom_id=5
           atom_name=' PO '
           alternate=' '
           residue_name="INT"
           chain_id='P'
           residue_id=1
           icode=" "
           x=coord[0]
           y=coord[1]
           z=coord[2]
           occ=0.00
           bfac=100.00
           free=6*' '
           seqid="POIN"
           symbol=" T"
           charge=" "
           state=-1
           cmd.read_pdbstr('%6s%5d %4s%1s%3s %1s%4d%1s   %8.3f%8.3f%8.3f%6.2f%6.2f%6s%4s %2s%2s'%\
                           (record_name,
                            atom_id,
                            atom_name,
                            alternate,
                             residue_name,
                             chain_id,
                             residue_id,
                             icode,
                             x,
                             y,
                             z,
                             occ,
                             bfac,
                             free,
                             seqid,
                             symbol,
                             charge
                                                ),name,state)
           try:
               cmd.show_as("nb_spheres",name)
           except AttributeError:
              print "cmd.as() has changed to cmd.show_as() in later and previous versions of PyMOL "
              #cmd.show_as("nb_spheres",name)
def get_axes(name="axes",origin="(0.0,0.0,0.0)",lenght="10.0",linewidth="3.0"):
        '''Create Axes.
        USAGE: get_axes(name="axes",origin="(0.0,0.0,0.0)",lenght="10.0",linewidth="1.0")
        PYMOL: axes name="axes",origin="(0.0,0.0,0.0)",lenght="10.0",linewidth="1.0"
                    name:  name of object
                    origin:  zero coordinates of axes or "box" for origin coordinates of selection min coordinates
                    lenght:  lenght of axes
                    linewidth: thickness of axes
        '''         
        if origin=="box":
         origin,(xmax,ymax,zmax)=cmd.get_extent(selection)
        try:
           linewidth=float(linewidth)
        except ValueError:
           print "Wrong linewidth input 1.0 used instead"
           linewidth=1.0
        if type(origin) is str:
            origin=_remove_brakets(origin)
            origin=origin.split(",")
            origin=tuple(map(float,origin))
        else:
           pass
        lenght=float(lenght)
        dic=cgo_dic()
        BEGIN=dic['BEGIN']
        TRIANGLES=dic['TRIANGLES']
        COLOR=dic['COLOR']
        SPHERE=dic['SPHERE']
        LINES=dic['LINES']
        NORMAL=dic['NORMAL']
        VERTEX=dic['VERTEX']
        END=dic['END']
        LINEWIDTH=dic['LINEWIDTH']
        a=[LINEWIDTH,linewidth,
           BEGIN,
           LINES,
           COLOR,0.0,1.0,0.0,
           VERTEX,
           0.0+origin[0],0.0+origin[1],0.0+origin[2],
           VERTEX,
           0.0+origin[0],lenght+origin[1],0.0+origin[2],
           COLOR,1.0,0.0,0.0,
           VERTEX,
           0.0+origin[0],0.0+origin[1],0.0+origin[2],
           VERTEX,
           lenght+origin[0],0.0+origin[1],0.0+origin[2],
           COLOR,0.0,0.0,1.0,
           VERTEX,
           0.0+origin[0],0.0+origin[1],0.0+origin[2],
           VERTEX,
           0.0+origin[0],0.0+origin[1],lenght+origin[2],
           END,
           COLOR,1.0,0.0,0.0,
           SPHERE,
           0.0+origin[0],0.0+origin[1],0.0+origin[2],0.2]
        print "Axes\n-----\n\tX :red\n\tY :green\n\tZ :blue"
        cmd.load_cgo(a,name)
def get_box_handler(selection="(sele)",app=None,action="medium",name="box",cushion="1.5",draw="full",coord="None",linewidth="3.0"):
     '''Get a plane using get_plane GUI.
     '''
     action=_remove_brakets(action)
     name=check_names(name)
     cushion=cushion
     draw=str(draw)
     linewidth=str(linewidth)
     if action=="set" and app:
      value=tkSimpleDialog.askstring('Box settings',
                                     'Specify [cushion] [draw full/lines]  [linewidth]',
                                                      parent=app.root)
      value=_remove_brakets(value)
      if value:
        value = value.split()
      else:
        return
      if len(value)==1:
         cushion=value[0]
      elif len(value)==2:# width / dpi
         cushion=value[0]
         draw=str(value[1])
      elif len(value)>=3:#width width_unit 
         cushion=value[0]
         draw=str(value[1])
         linewidth=str(value[2])
      else:
        print "Wrong input. Set at least size."
	return
      try:
        float(cushion)
      except ValueError:
        print "Cushion has to be a number."
	return
      units=["full","lines"]
      if not draw.lower().strip() in units:
            print "Draw has to be either %s "%(" or ".join(units)) 
	    return
      try:
        float(linewidth)
      except ValueError:
        print "Linewidth has to be a number."
	return
     elif action=="small":
	cushion=1
        draw="full"
        linewidth=3
     elif action=="medium":
        cushion=5
        draw="full"
        linewidth=3
     elif action=="large":
        cushion=50
        draw="full"
        linewidth=3
     get_box(selection=selection,name=name,cushion=cushion,draw=draw,linewidth=linewidth)
def get_box(name="box",selection="all",cushion="1.5",draw="full",coord="None",linewidth="3.0"):
        '''Create a box.
        USAGE: get_box(name="box",
                          selection="all",
                          cushion=0.0,
                          draw="full",
                          coord="None",
                          linewidth="1.0"
                          )
        PYMOL: box name=box,selection=all,cushion=2.0,draw=full,coord=None,linewidth=1.0
                  name:       name of the cgo box
                  selection:  selection for min and max of box calculation
                  cushion  :  extend box with cushion in Angstroms each side
                              or each side separately by "(xcmin,ycmin,zcmin,xcmax,ycmax,zcmax)"
                  draw:     lines:       draw a box with lines
                            full:        draw a full box with each side separated
                                         to hide box walls separately
                  coord    :  if coord is given as "(xmin,ymin,zmin,xmax,ymax,zmax)"
                              selection will be ignored
                  linewidth: 1.0 : in case of box with lines define linewidth
        '''
	draw=_remove_brakets(draw)
        name=_remove_brakets(name)
        dic=cgo_dic()
        try:
           cushion=float(cushion)
           cushion=(cushion,cushion,cushion,cushion,cushion,cushion)
        except ValueError:
             try:
               cushion=_remove_brakets(cushion)
               cushion=cushion.split(",")
               cushion=tuple(map(float,cushion))
             except ValueError:
                pass
             if len(cushion)==6:         
               pass
             else:
              cushion=(0.0,0.0,0.0,0.0,0.0,0.0)
        linewidth=float(linewidth)      
        if coord=="None":
         (xmin,ymin,zmin),(xmax,ymax,zmax)=cmd.get_extent(selection)
         (xmin,ymin,zmin),(xmax,ymax,zmax)=(xmin-cushion[0],ymin-cushion[1],zmin-cushion[2]),(xmax+cushion[3],ymax+cushion[4],zmax+cushion[5])
        else:
           if type(coord) is str:
            coord=_remove_brakets(coord)
            coord=coord.split(",")
            coord=tuple(map(float,coord))
           else:
              pass
           xmin,ymin,zmin,xmax,ymax,zmax = coord
           xmin,ymin,zmin,xmax,ymax,zmax = xmin-cushion[0],ymin-cushion[1],zmin-cushion[2],xmax+cushion[3],ymax+cushion[4],zmax+cushion[5]
        edge01=(xmax,ymax,zmax)    
        edge02=(xmin,ymax,zmax)
        edge03=(xmin,ymax,zmin)
        edge04=(xmax,ymax,zmin)
        edge05=(xmax,ymin,zmin)
        edge06=(xmax,ymin,zmax)
        edge07=(xmin,ymin,zmax)
        edge08=(xmin,ymin,zmin)
        normx=(-1.0,0.0,0.0)
        normx1=(1.0,0.0,0.0)
        normy=(0.0,-1.0,0.0)
        normy1=(0.0,1.0,0.0)
        normz=(0.0,0.0,-1.0)
        normz1=(0.0,0.0,1.0)
        if draw=="lines":
            BEGIN=dic['BEGIN']
            LINES=dic['LINES']
            LINEWIDTH=dic['LINEWIDTH']
            VERTEX=dic['VERTEX']
            END=dic['END']
            box= [
            LINEWIDTH, linewidth,
            BEGIN,
            LINES,
            VERTEX, edge01[0], edge01[1], edge01[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            VERTEX, edge03[0], edge03[1], edge03[2], 
            VERTEX, edge04[0], edge04[1], edge04[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            VERTEX, edge01[0], edge01[1], edge01[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            VERTEX, edge01[0], edge01[1], edge01[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            VERTEX, edge08[0], edge08[1], edge08[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            VERTEX, edge06[0], edge06[1], edge06[2],        
            VERTEX, edge07[0], edge07[1], edge07[2],
            VERTEX, edge07[0], edge07[1], edge07[2],        
            VERTEX, edge08[0], edge08[1], edge08[2],
            VERTEX, edge08[0], edge08[1], edge08[2],        
            VERTEX, edge05[0], edge05[1], edge05[2],
            END     
            ]
            cmd.load_cgo(box,name)
        elif draw=="full":
            box=[]
            BEGIN=dic['BEGIN']
            TRIANGLES=dic['TRIANGLES']
            COLOR=dic['COLOR']
            SPHERE=dic['SPHERE']
            LINES=dic['LINES']
            NORMAL=dic['NORMAL']
            VERTEX=dic['VERTEX']
            END=dic['END']
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge01[0], edge01[1], edge01[2],
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            NORMAL,normy[0],normy[1],normy[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            END     
            ]
            box.append(a)
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge01[0], edge01[1], edge01[2],
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            NORMAL,normx[0],normx[1],normx[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            END     
            ]
            box.append(a)
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge08[0], edge08[1], edge08[2],
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            NORMAL,normz1[0],normz1[1],normz1[2],
            VERTEX, edge04[0], edge04[1], edge04[2],
            END     
            ]
            box.append(a)
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge03[0], edge03[1], edge03[2],
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            NORMAL,normx1[0],normx1[1],normx1[2],
            VERTEX, edge08[0], edge08[1], edge08[2],
            END     
            ]
            box.append(a)
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge01[0], edge01[1], edge01[2],
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge02[0], edge02[1], edge02[2],
            NORMAL,normz[0],normz[1],normz[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            END     
            ]
            box.append(a)
            a=[
            BEGIN, TRIANGLES,
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge06[0], edge06[1], edge06[2],
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge05[0], edge05[1], edge05[2],
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge08[0], edge08[1], edge08[2],
            NORMAL,normy1[0],normy1[1],normy1[2],
            VERTEX, edge07[0], edge07[1], edge07[2],
            END     
            ]
            box.append(a)
            color=["wheat",
                   "lightpink",
                   "palecyan",
                   "darksalmon",
                   "gray90",
                   "palegreen"]
            for i,(item,color) in enumerate(zip(box,color)):
                cmd.load_cgo(item,name+"_%d"%(i+1))
                cmd.color(color,name+"_%d"%(i+1))
def fast_sym(selection='',cutoff=4.0,delete_sym=1,use=1,app=None):
                '''Create a view of symmetry involved interactions.
                AUTHOR:
                        Georg Steinkellner, 11/2006
                USAGE: fast_sym selection,cutoff=4.0,delete_sym=1
                                selection:   object for symmetry calculation
                                cutoff:      distance cutoff
                                delete_sym:  delete previous sym files ("s_*")
                '''
                selection=_remove_brakets(selection)
                if cutoff=="set" and app:
                        cutoff=tkSimpleDialog.askstring('Input Symetry Distance',
                                                      'Please specify a Distance e.g 4.25',
                                                      parent=app.root)
                if cutoff:
                        try:
                                cutoff=float(cutoff)
                                use=int(use)
                                delete_sym=int(delete_sym)
                        except:
                                print "Wrong input"
                                print fast_sym.__doc__
                        else:
                                cryst=Sym()
                                if delete_sym==1:#delete previous symetry objects
                                        print delete_sym
                                        del_sym()
                                if selection=="sele":# selection=="sele":
                                   if cmd.get_object_list(selection) is None:
                                     print "No structure in selection (sele). Select at least 1 atom" 
                                     print "in a structure."
                                     return
                                   if len(cmd.get_object_list(selection)) > 1:
                                      print "MORE THAN ONE OBJECT IN SELECTION"
                                      print "FIRST OBJECT IS USED"
                                   try:
                                          selection=selection+" and "+cmd.get_object_list(selection)[0]#just use first hit
                                   except IndexError:
                                           print "No Object enabled"
                                           print "Enable a Object"
                                           selection=''   
                                           return
                                elif selection=='':#get first object
                                        names=cmd.get_names(enabled_only=1)
                                        #selection=' or '.join(names)
                                        if len(names) > 1:
                                                print "MORE THAN ONE OBJECT IS ENABLED"
                                                print "FIRST OBJECT IS USED"
                                        elif len(names) == 0:
                                                print "Load a structure into PyMOL first."
                                                return
                                        try:
                                          selection=names[0]#just use first hit
                                        except IndexError:
                                           print "No Object enabled"
                                           print "Enable a Object"
                                           selection=''
                                else:
                                   print "Load a structure into pymol. No structure selected"
                                   return
                                mol=cryst.get_model(selection)
                                if mol: 
                                        if len(mol.atom) ==0:
                                                        print "***\n*** No Atoms in selection\n***"
                                        elif use==1:    
                                                        original=selection
                                                        cmd.hide("surface")
                                                        sym_select=cryst.create_sym(cutoff=cutoff,selection=selection)
                                                        set_surface(mode="symmetry")
                                                        org_select=cryst.create_org(cutoff=cutoff,sele="s_*")
                                                        print org_select
                                                        cryst.show_org(select_name=org_select,show_name="surface",cutoff=cutoff,surf_transparency=0.5)
                                                        cryst.show_org(select_name=org_select,show_name="stick",cutoff=cutoff,surf_transparency=0.5)
                                                        cryst.show_sym(select_name=sym_select,show_name="surface",cutoff=cutoff,surf_transparency=0.5,object="s_*")
                                                        cryst.show_sym(select_name=sym_select,show_name="stick",cutoff=cutoff,surf_transparency=0.5,object="s_*")
                                        cmd.delete("tmp_org")
import tkSimpleDialog
import tkMessageBox
import tkFileDialog
from Tkinter import *
import Tkinter
import Pmw
class multi_pdb:
    '''Multiple input load. 
    '''
    filedir='.'
    def __init__(self,app,case=1,message=None):
        self._message=message
        try:
           self._app=app.root
        except:
           self._app=app
        self.filedir = self.__class__.filedir
        self.filename = []
        self.filetypes=[('PDB File','.pdb'),
                        ('PDB File','.ent'),
                        ('Modeller PDB File','.B9999*.pdb'),
                        ('Modeller PDB File','.B9999*_fit.pdb'),
                        ('Delphi PHI File','.phi') ,
                        ('all','*')                                                           ,
                        ]
    def _map(self,filenamelist):
       '''load Electon density map.
       '''
       pass
    def _casox_map(self,filenamelist,repr="mesh"):
      '''Load a CaSoX map (xplor format) and generate different level views.
       USAGE: _casox_map(filenamelist,repr="mesh")
                   repr:   "mesh"   :show levels in mesh format
                           "surf"   :show levels in surf format
       ADDITIONAL: this will create a map and different mesh representations
                   in different colors depending on casox ligsite values
                   level 0 --> "white"       --> casox ligsite level 1
                   level 1 --> "bluewhite"   --> casox ligsite level 2
                   level 2 --> "palecyan"    --> casox ligsite level 3
                   level 3 --> "lightteal"   --> casox ligsite level 4
                   level 4 --> "lightorange" --> casox ligsite level 5
                   level 5 --> "orange"      --> casox ligsite level 6
                   level 6 --> "red"         --> casox ligsite level 7 (complete closed cavity)
                   level-1 -->  "blue"       --> casox protein level 0 grid hull                   
      '''
      if len(filenamelist)<>1:
          print "Please select just one CaSOX map."
          file_path=norm_path(filenamelist[0])
          self.__class__.filedir=file_path.dirname
          return
      else:
         file_path=norm_path(filenamelist[0])
         objcasox=file_path.splitname+"_map"
         self.open_filename(file_path.all,obj=objcasox,format="xplor")
         self.__class__.filedir=file_path.dirname
      if cmd.get_type(objcasox)=="object:map":
         levels=[-1,6,5,4,3,2,1,0]
         colors=["blue","red","orange","lightorange","lightteal","palecyan","bluewhite","white"]
         print "CaSoX Map"
         print "--------------"
         print "Loaded by BNI-Tools v.%s"%__version__
         print "Map: %s "%(objcasox)
         print "Levels for Ligsite 1 - 7(closed cavity)"
         cmd.feedback("disable","cmd","everything")
         for level,color in zip(levels,colors):
            objmesh=file_path.splitname+"_%d"%(level+1)
            if repr=="mesh":
               cmd.isomesh(objmesh,objcasox,1.0)
            elif repr=="surf":
               cmd.isosurface(objmesh,objcasox,1.0)
               cmd.hide("surface",objmesh)
            cmd.isolevel(objmesh,float(level))
            cmd.color(color,objmesh)
            print "%15s : %15s"%(objmesh,color)
         print "--------------"
         cmd.feedback("enable","cmd","everything")
    def _delphi(self,filenamelist):
       '''Load phi and pdb file to create a delphi surface.
       '''
       if len(filenamelist)>2:
          print "Select .phi or .dx map and corresponding .pdb file."
          file_path=norm_path(filenamelist[0])
          self.__class__.filedir=file_path.dirname
          return
       else:
          objphi="e_lvl"
          map=0
          for item in filenamelist:
             file_path=norm_path(item)
             if file_path.splitext.lower()==".phi" or file_path.splitext.lower()==".dx":
                   objmap=file_path.splitname+"_map"
                   objmapext=file_path.splitext.lower()
                   objphi=file_path.splitname
                   self.open_filename(item,obj=objmap)
                   map=1
             else:
                objpdb=file_path.splitname
                self.open_filename(item,obj=objpdb)
          self.__class__.filedir=file_path.dirname
          if map==1 and cmd.get_type(objmap)=="object:map":
           objramp=objphi+"_ramp"
           cmd.ramp_new(objramp,objmap,(-15,0,+15))
           if objmapext==".phi":
               print "DELPHI PHI Map"
           elif objmapext==".dx":
               print "DX Map"
           print "--------------"
           print "Loaded by BNI-Tools v.%s"%__version__
           print "Map    : %s"%objmap
           print "Ramp   : %s"%objramp
           if len(filenamelist)==2:
             cmd.set("surface_color",objramp,objpdb)
             print "File   : %s"%objpdb
           else:
              print "Load Pdb file as object into pymol and"
              print "set surface_color,%s,(object)"%objramp
           print "--------------"       
           print "To change ramp values use str+mid mouse button on ramp"
           print "or use"
           print "ramp_new %s,%s,(min,mid,max)"%(objramp,objmap)
           print "to set ramp values."
          else:
             print "No Map loaded ... (extention has to be .phi or .dx)"
    def _modeller(self,filenamelist):
       '''Load multiple modeller files into Pymol.
       '''
       filenamelist=list(filenamelist)
       filenamelist.sort()
       filenamelist=tuple(filenamelist)
       for item in filenamelist:
            file_path=norm_path(item)
            self.filename.append(file_path.basename)
       self.__class__.filedir=file_path.dirname
       objfunc=[]
       seqid=[]
       modvers=[]
       filelist=[]
       object=[]
       for files in self.filename:
            file_path=norm_path(self.__class__.filedir,files)
            filelist.append(file_path.all)
            obj=file_path.splitname
            if ".B9999" in obj: 
              obj=obj.split(".B9999")
              obj1=obj[0]
              try:
               obj2=obj[1]
              except IndexError:
               obj2=""
               obj=obj1
              try:
               obj2=int(obj2)
              except ValueError:
               pass
              obj=obj1+'_'+"M"+str(obj2)
            elif ".BL" in obj:
              obj=obj.split(".BL")
              obj1=obj[0]
              try:
               obj2=obj[1][:-4]
              except IndexError:
               obj2=""
               obj=obj1
              try:
               obj2=int(obj2)
              except ValueError:
               pass
              obj=obj1+'_'+'L'+str(obj2)
            object.append(obj)
            try:
              fileobj=open(file_path.all,"r")
            except IOError:
               print "Cannot read modeller file  %s "%file_path.basename
               object.pop() 
               continue
            else:
               sid=0.0
               objf=0.0
               while 1:
                  a=fileobj.readline()
                  if not a:
                     break
                  if "ATOM" in a[:4] or "HETATM" in a[:6]:
                     break
                  if "EXPDTA" in a[:6] and "MODELLER" in a:
                     continue
                  elif "REMARK" in a[:6] and "MODELLER OBJECTIVE FUNCTION" in a:
                     try:
                        objf=float(a.split()[-1])
                     except (IndexError,ValueError):
                        print "Cannot parse Objective Funktion for %s"%file_path.basename
                        pass
                  elif "REMARK" in a[:6] and "MODELLER BEST TEMPLATE" in a:
                     try:
                        sid=float(a.split()[-1])
                     except (IndexError,ValueError):
                        print "Cannot parse SEQ ID for %s"%file_path.basename
                        pass
               objfunc.append(objf)
               seqid.append(sid)
       s=zip(objfunc,filelist,object,seqid)
       s.sort()
       print "Modeller Files"
       print "--------------"
       print "Loaded by BNI-Tools v.%s"%__version__
       print "Sorted by Modeller Objective Function"
       print "Name ;Objective Function; Seq-ID"
       for objfunc,item,obj,seqid in s:
          if "%.2f"%objfunc=="0.00":
             self.open_filename(item,obj=obj)
          else:
             self.open_filename(item,obj=obj,title="%.2f"%objfunc)
          print "%-15s ; %.2f ;  %.2f%%"%(obj,objfunc,seqid)
       print "--------------"
    def _amber(self,filenamelist):
       '''Load multiple amber pdb files with info file and energy.
       '''
       filenamelist=list(filenamelist)
       filenamelist.sort()
       filenamelist=tuple(filenamelist)
       for item in filenamelist:
            file_path=norm_path(item)
            self.filename.append(file_path.basename)
       self.__class__.filedir=file_path.dirname
       filelist=[]
       print "Amber Files"
       print "--------------"
       print "Loaded by BNI-Tools v.%s"%__version__
       info_file_ext=".info"   
       print "_"*60
       print "%-40s:%-20s"%("Name","Energy")
       print "-"*60               
       for files in self.filename:
            file_path=norm_path(self.__class__.filedir,files)
            filelist.append(file_path.all)
            obj=file_path.splitname
            try:
              self.open_filename(file_path.all,obj=obj)
            except IOError:
               print "Cannot open file %s"%file_path.basename
               continue
            try:
              info_file=norm_path(file_path.dirname,file_path.splitname+info_file_ext)
              if  os.path.isfile(info_file.all):
                   fileobj=open(info_file.all,"r")
              elif "_min." in file_path.basename:#try to find info file if not same name as pdb file
                   info_name=file_path.splitname.split("_min")[0]
                   info_file=norm_path(file_path.dirname,info_name+info_file_ext)
                   fileobj=open(info_file.all,"r")
              else:                 
                 print "%s has no amber .info file in same directory."%(file_path.basename)
                 print "Therefore no energy is read in."
                 continue
            except IOError:
               print "Cannot read amber file  %s "%file_path.basename
               continue
            else:
               energy=0.0
               energy_sign="NSTEP"
               remark_text=[]
               begin_read=0
               begin_energy=0
               while 1:
                  a=fileobj.readline()
                  if not a:
                     break
                  if begin_energy == 1:
                     try:
                       energy = float(a.split()[1])
                     except IndexError,ValueError:
                       energy=0.0
                     begin_energy=0
                  if energy_sign in a:
                     begin_read=1
                     begin_energy=1
                     remark_text.append(a)
               fileobj.close()
               print "%-40s:%-20.3f"%(obj,energy)
               try:
                    cmd.set_title(obj,-1,"%-5.1f"%(energy))
               except:
                    pass
       print "-"*60
    def _autodock(self,filenamelist):
       '''Load multiple autodock MODEL files.
       '''
       filenamelist=list(filenamelist)
       filenamelist.sort()
       filenamelist=tuple(filenamelist)
       for item in filenamelist:
            file_path=norm_path(item)
            self.filename.append(file_path.basename)
       self.__class__.filedir=file_path.dirname
       filelist=[]
       print "AutoDock Files"
       print "--------------"
       print "Loaded by BNI-Tools v.%s"%__version__
       print "_"*60
       print "%-30s:%-30s"%("Name","Nr. of Models")
       print "-"*60               
       for files in self.filename:
            file_path=norm_path(self.__class__.filedir,files)
            filelist.append(file_path.all)
            obj=file_path.splitname
            try:
              self.open_filename(file_path.all,obj=obj)
            except IOError:
               print "Cannot open file %s"%file_path.basename
               continue
            try:
              fileobj=open(file_path.all,"r")
            except IOError:
               print "Cannot read autodock file  %s "%file_path.basename
               continue
            else:
                cluster=[]
                cluster_sign="Number of conformations in this cluster"
                cluster_split="="
                energy=[]
                energy_sign="Estimated Free Energy of Binding"
                energy_split="="
                remarks=[]
                remark_text=[]
                model_state=[]
                model_state_id=0
                model_id = []
                begin_read=0
                while 1:
                   a=fileobj.readline()
                   if not a:
                      break
                   if "ENDMDL" in a[:6]  and begin_read==1:
                      begin_read=0
                      remarks.append(remark_text)
                      remark_text=[]
                      continue
                   if "REMARK" in a[:6] or "USER" in a[:4] and begin_read==1:
                      try:
                         remark_text.append(a.split()[1])
                      except IndexError:
                         remark_text.append(" ")
                      if cluster_sign in a:
                         try:
                             cluster.append(int(a.split(cluster_split)[1].split()[0]))
                         except IndexError:
                             cluster.append(0)
                      if energy_sign in a:
                         try:
                            energy.append(float(a.split(energy_split)[1].split()[0]))
                         except IndexError:
                            energy.append(0.0)
                      continue      
                   if "MODEL" in a[:5] :
                      try:
                        model_id.append(int(a.split()[-1]))
                      except IndexError:
                         model_id.append(0)
                      model_state_id+=1
                      model_state.append(model_state_id) 
                      begin_read=1
                fileobj.close()
                print "%-30s:%-30d"%(obj,len(model_state))
                print "-"*60
                print "%-5s%-4s:%-4s:%-10s :%-10s"%(" ","Nr","Id","Energy","Cluster")
                for item in zip(model_state,model_id,energy,cluster): 
                  print "     %-4d:%-4d:%10.3f : %-10d"%item
                  try:
                     cmd.set_title(obj,item[0],"%5.2f [%2d]"%(item[2],item[3]))
                  except:
                     pass
                print "-"*60
                if file_path.splitext == ".dlg":
	         print "DLG File loaded (just best docking mode in cluster is shown.)"
                 # additional read all docked models
        #s=zip(objfunc,filelist,object,seqid)
        #s.sort()
    def _oneobject(self,filenamelist,sorting=1):
       '''Load multiple files into one object.
          Object is named by first filename.
          If yasara table (.tbl) is present in selection try to read energies.
	  sorting: 1: sort by filename
       '''
       filenamelist=list(filenamelist)
       if sorting==1:
         filenamelist.sort()
       tablename=None
       try:
         table_index=[filenamelist.index(a) for a in filenamelist if "tab" in norm_path(a).splitext]
	 if table_index:
           for tab_index in table_index:
             tablename=filenamelist.pop(tab_index)
           tablename=norm_path(tablename)
       except IndexError:
         pass
       try:
          object_name=norm_path(filenamelist[0]).splitname
       except IndexError:
          object_name="obj1"
       filenamelist=tuple(filenamelist)
       for item in filenamelist:
            file_path=norm_path(item)
            self.filename.append(file_path.basename)
       self.__class__.filedir=file_path.dirname
       for files in self.filename:
            file=norm_path(self.__class__.filedir,files).all
            self.open_filename(file,obj=object_name)
       if tablename:  
            try:
              fileobj=open(tablename.all,"r")
            except IOError:
               print "Cannot read table file  %s "%tablename.basename
            else:
               begin_read=0
               energy=[]
               while 1:
                  a=fileobj.readline()
                  if not a:
                     break
                  if begin_read==1:
                    energy.append(a.strip().split())
                  if "snapshot" in a.strip().lower():
                     print "Sanapshot   Energy"
                     print "------------------"
                     begin_read=1
                     continue
               fileobj.close()
               energy=[e for e in energy if len(e)==2]
               for i,(snap,value) in enumerate(energy): 
                 print "%-5s:%-30s"%(snap,value)
                 try:
                    cmd.set_title(object_name,i+1,"%s:%s"%(snap,value))
                 except:
                    pass
               print "-"*60
    def _all(self,filenamelist):
       '''Load multiple files.
       '''
       filenamelist=list(filenamelist)
       filenamelist.sort()
       filenamelist=tuple(filenamelist)
       for item in filenamelist:
            file_path=norm_path(item)
            self.filename.append(file_path.basename)
       self.__class__.filedir=file_path.dirname
       for files in self.filename:
            file=norm_path(self.__class__.filedir,files).all
            self.open_filename(file,obj="None")
    def fetchFileName(self,ext="None"):
        '''Get multiple filenmes loaded into Pymol with special filename handling.
        USAGE: fetchFileName(ext="None")
                 ext:   "None"     :load files with filename extention
                        "modeller" :load modeller files sorted
                                    by modeller objective function
                                    and rename filenames .B9999000X to _X
                        "autodock" :load autodock pdb files (docked MODELs)
                                    with energy and cluster in title
                        "ambermin" :load amber minimization pdb with energy
                                    in .info files
                        "delphi"   :load delphi map (*.phi) and pdb file for surface representation
                                    or abps map (*.dx) and pdb file for surface representation
                        "casox"    :load CaSoX Map (xplor like Soft Ligsite Cavity Explorer)
        '''       
        if ext == "None":
           filetypes=self.filetypes
        elif ext=="oneobject":
           filetypes=self.filetypes
        elif ext=="modeller":
           filetypes=[('Modeller PDB File','.B9999*.pdb'),('all','*')]
        elif ext=="autodock":
           filetypes=[('AutoDock docked PDB','.dlg.pdb'),('AutoDock docked PDB','.pdb'),('AutoDock DLG file','.dlg'),('AutoDock DLG file','.pdb'),('all','*')]
        elif ext=="ambermin":
           filetypes=[('Amber minimization','.pdb'),('all','*')]
        elif ext=="delphi":
           filetypes=[('DelPhi PHI Map','.phi'),('DelPhi PHI Map','.pdb'),('ABPS dx Map','.pdb'),('ABPS dx Map','.dx'),('all','*')]
        elif ext=="casox":
           filetypes=[('CaSoX Map','.omap'),('CaSoX Map','.map'),('all','*')]
        filenamelist=tkFileDialog.askopenfilenames( title='Open Files',
                                                    initialdir=self.__class__.filedir,
                                                    filetypes=filetypes,
                                                    )
        self.filename=[]
        self.filedir=self.__class__.filedir
        if filenamelist:
         if ext=="None":
            self._all(filenamelist)
         elif ext=="modeller":
            self._modeller(filenamelist)
         elif ext=="autodock":
            self._autodock(filenamelist)
         elif ext=="ambermin":
            self._amber(filenamelist)
         elif ext=="delphi":
            self._delphi(filenamelist)
         elif ext=="casox":
            self._casox_map(filenamelist)
         elif ext=="oneobject":
            self._oneobject(filenamelist)
    def open_filename(self,file,obj="None",state="None",format="None",finish="None",discrete="None",title="None",allinone="0"):
       '''Open file in pymol.
       USAGE: open_filename(file,
                            obj="None",
                            state="None",     # not implemented
                            format="None",    # specify format
                            finish="None",    # not implemented
                            discrete="None",  # not implemented
                            title="None"      # write title
                            allinone="0")     # all in one object (first object name) with different states 
       '''
       if obj=="None":
          obj=norm_path(file).splitname
       else:
          pass
       if state=="None":
           state=-1
       if format=="None":
          cmd.load(file,object=obj,state=state)
       else:
          cmd.load(file,object=obj,format=format,state=state)
          #print "loaded",file,obj,format,state
       if title=="None":
          title=""
       else:
          cmd.set_title(obj,state,title)
def norm_path(pathname,*join,**abs):
       '''Create normalized path and join path items.
       USAGE: norm_path(pathname
                        [,*join]   # join "join" strings to path
                        [,**abs=1] # 1:  create absolute path
                                     0:  use relative path
                        )
       RETURNS: Object with obj.all
                               .dirname
                               .basename
                               .splitname
                               .splitext
       ADDITIONAL: e.g. 'C:/user/something\\new/'
                              .all    'C:\\user\\something\\new'
                              .dirname 'C:\\user\\something'
                              .basename 'new'
                              .splitname 'new'
                              .splitext  ''
                        'C:/user/something\\new/file.txt'
                              .all    'C:\\user\\something\\new\\file.txt'
                              .dirname 'C:\\user\\something\\new'
                              .basename 'file.txt'
                              .splitname 'file'
                              .splitext  '.txt'
       '''
       if abs.get('abs',1)==1:
         whole=os.path.normpath(os.path.abspath(os.path.join(pathname,*join)))
       else:
         whole=os.path.normpath(os.path.join(pathname,*join))     
       path=os.path.dirname(whole)
       file_name=os.path.basename(whole)
       pref,ext=os.path.splitext(file_name)
       class NormPath:
              def __init__(self):
                     self.all=whole
                     self.dirname=path
                     self.basename=file_name
                     self.splitname=pref
                     self.splitext=ext
       output=NormPath()
       return  output   
class jump:
        '''
        '''
        u=None
        def __init__(self,selection,id,aminoacid):
           if id=="first" and selection:
              self.u=AA()
              self.u.add_atom_model(selection)
           elif id=="n" and self.u:
                self.u.next(aminoacid)
           elif id=="b" and self.u:
                self.u.back(aminoacid)
           else:
                print "no selection"
def jump_aa(id="first",aminoacid="NQH",selection=None):
        '''
        '''
        jump(selection,id,aminoacid)
class AA:
    '''
    USAGE: AA().add_atom_model(selection)
               .next(AA)
               .back(AA)
    '''
    def __init__(self):
        self.chain ={}
        self.chain_id=[]
        self._nextaa=0
        self._nextch=0
        self._nextback=0
        self.atom_model=[]
        self._oldchain=[]
        self.__count=0
        self.__foundaa=0
    def add_atom_model(self,selection):
        selection=_remove_brakets(selection)
        cmd.sort()
        try:
            #TODO remove waters and hetatms from selection only keep protein
            selection=selection+" and not solvent and not hetatm"
            self.atom_model=cmd.get_model("%s"%selection)
            self.atom_model.sort()
        except:
            print "Cannot create model from selection %s"%selection
        else:
            self.add_atoms2chain(chain=selection,atomlist=self.atom_model.get_residues())
    def add_atoms2chain(self,chain="all",atomlist=[]):
        self.chain[chain]=atomlist
        self.chain_id.append(chain)
    def back_aa(self,chain_id=None):
        if chain_id is None:
            try:
                chain_id=self.chain_id[self._nextch]
                list=self.chain[chain_id]
            except IndexError:#no more chains
                list=self.chain[chain_id]
        else:
            try:
                list=self.chain[chain_id]
            except KeyError:
                print "No such Chain:%s"%chain_id
                return (chain_id,[])
            try:
              out=(chain_id,list[self._nextaa])
            except IndexError:
              print "End of Chain %s"%chain_id
              self._nextaa=0
              out=(chain_id,list[self._nextaa])
            self._nextaa-=1
            return out
        try:
          out=(chain_id,list[self._nextaa])
        except IndexError:
          try:
              self._nextch-=1
              next_ch=self.chain_id[self._nextch]
              print "Chain %s"%next_ch
              self._nextaa=0
              list=self.chain[next_ch]
              out=(next_ch,list[self._nextaa])
              self._nextaa-=1
              return out
          except IndexError:
              self._nextch=0#begin on first
              print "End of Chain"
              self._nextaa=0
              out=(chain_id,list[self._nextaa])
              self._nextaa-=1
              return out
        self._nextaa-=1
        return out
    def next_aa(self,chain_id=None):
        '''Get next amino acid atom numbers.
        USAGE: next_aa(chain_id=None):
                    chain_id: can be specified like "A" or if loaded
                              a model with the name of the
                              model e.g. "1CRN" or "mymodel"
                              if no chain_id is specified it
                              goes over all chain- ids loaded
        RETURNS: (chain_id,[(0,9),(9,12),(12,17)])
                    (0,9) : atom ids (+1!) of amino acid 1
                                e.g 
                    (9,12): atom ids (+1!) of amino acid 2
        '''
        if chain_id is None:
            try:
                chain_id=self.chain_id[self._nextch]
                list=self.chain[chain_id]
            except IndexError:#no more chains
                list=self.chain[chain_id]
        else:
            try:
                list=self.chain[chain_id]
            except KeyError:
                print "No Sequence:%s"%chain_id
                return (chain_id,[])
            try:
              out=(chain_id,list[self._nextaa])
            except IndexError:
              print "End of Sequence %s"%chain_id
              self._nextaa=0
              out=(chain_id,list[self._nextaa])
            self._nextaa+=1
            return out
        try:
          out=(chain_id,list[self._nextaa])
        except IndexError:
          try:
              self._nextch+=1
              next_ch=self.chain_id[self._nextch]
              print "Sequence: %s"%next_ch
              self._nextaa=0
              list=self.chain[next_ch]
              out=(next_ch,list[self._nextaa])
              self._nextaa+=1
              return out
          except IndexError:
              self._nextch=0#begin on first
              print "End of all Sequences"
              self._nextaa=0
              out=(chain_id,list[self._nextaa])
              self._nextaa+=1
              return out
        self._nextaa+=1
        return out
    def back(self,aa="all",animate=1):
        '''Go to previous amino acid in sequence.
        USAGE: back(aa="all",animate=1)
                aa:    "all"        :go to previous amino acid
                       "NQH"        :go to previous N or Q or H (for flipping)
                       "HIS"        :go to previous amino acid HIS (can be any 3 letter code)
                       "HIS;ALA;GLY":go to previous HIS or ALA or GLY
                animate: 1: animate
                         0: dont animate just go back one AA
        '''    
        if self._nextback==1:
            self._nextaa-=2
        if aa=="all":
            sele,rlist=self.back_aa()
        else:
            if aa=="NQH":
                aa_list=["ASN","GLN","HIS","HID","HIP","HIE"]
            else:
                aa_list=aa.split(";")
            while 1:
                sele,rlist=self.back_aa()
                print "byresi(%s`%s) and (r. %s)"%(sele,rlist[0]+1,",".join(aa_list))
                cmd.select("_aa","byresi(%s`%s) and (r. %s)"%(sele,rlist[0]+1,",".join(aa_list)))
                print cmd.count_atoms("_aa",quiet=0)
                if cmd.count_atoms("_aa",quiet=0) > 0:
                    self.__foundaa=1
                    break
                elif self.__foundaa==0:#break the search if no aa in sequence was found
                    self.__count+=1
                    if self.__count > len(self.atom_model.get_residues()):
                        self.__count=0
                        print "%s not found in Sequence"%aa
                        return
                        break
        rlist1,rlist2=rlist
        print (sele,rlist)
        try:
            cmd.hide("sticks","(current)")
        except:
            pass
        u="|%s`"%sele
        print "(%s)"%(sele+'`'+u.join([str(i) for i in range(rlist1+1,rlist2+1)]))
        cmd.select("current","(%s)"%(sele+'`'+u.join([str(i) for i in range(rlist1+1,rlist2+1)])))
        #cmd.center("(current)")
        #cmd.zoom("(current)")
        cmd.orient("(current)",animate=animate)
        cmd.show("sticks","(current) and not n. C,O,N")
        self._nextback=-1
    def next(self,aa="all",animate=1):
        '''Go to next amino acid in sequence.
        USAGE: next(aa="all",animate=1)
                aa:    "all"        :go to next amino acid
                       "NQH"        :go to next N or Q or H (for flipping)
                       "HIS"        :go to next amino acid HIS (can be any 3 letter code)
                       "HIS;ALA;GLY":go to next HIS or ALA or GLY
                animate: 1: animate
                         2: no animation just go to next AA
        '''        
        if self._nextback==-1:
            self._nextaa+=2
        if aa=="all":
            sele,rlist=self.next_aa()
        else:
            if aa=="NQH":
                aa_list=["ASN","GLN","HIS","HID","HIP","HIE"]
            else:
                aa_list=aa.split(";")
            while 1:
                sele,rlist=self.next_aa()
                #print "byresi(%s`%s) and (r. %s)"%(sele,rlist[0]+1,",".join(aa_list))
                cmd.select("_aa","byresi(%s`%s) and (r. %s)"%(sele,rlist[0]+1,",".join(aa_list)))
                #print cmd.count_atoms("_aa",quiet=0)
                if cmd.count_atoms("_aa",quiet=0) > 0:
                    self.__foundaa=1
                    break
                elif self.__foundaa==0:#break the search if no aa in sequence was found
                    self.__count+=1
                    if self.__count > len(self.atom_model.get_residues()):
                        self.__count=0
                        print "%s not found in Sequence"%aa
                        return
                        break
        rlist1,rlist2=rlist
        print (sele,rlist)
        try:
            cmd.hide("sticks","(current)")
        except:
            pass
        u="|%s`"%sele
        print "(%s)"%(sele+'`'+u.join([str(i) for i in range(rlist1+1,rlist2+1)]))
        cmd.select("current","(%s)"%(sele+'`'+u.join([str(i) for i in range(rlist1+1,rlist2+1)])))
        #cmd.center("(current)")
        #cmd.zoom("(current)")
        cmd.orient("(current)",animate=animate)
        cmd.show("sticks","(current) and not n. C,O,N")
        self._nextback=1
